import mc.io.bvh.LODBVHParser;
import mc.io.bvh.ParseException;
import mc.model.AnimationModel;
import mc.model.ModelTimer;
import mc.model.BasicAnimationModel;
import mc.model.SkeletonHandle;
import mc.model.DummyTranslationDriver;
import mc.view.ViewFrame;
import mc.view.TreeModelView;
import mc.view.GLModelView;
import mc.view.Camera;
import mc.view.cameras.*;
import mc.model.ModelException;
import mc.model.euler.EulerAnglesDriverFactory;
import mc.model.quart.QuartenionDriverFactory;
import mc.model.map.MapDriverFactory;
import mc.model.DriverFactory;

import com.sun.java.util.collections.*;
import java.io.*;
import java.awt.event.*;

public class TestView2
{
    public static final String usage = "Usage: TestView [-i interpolator] [-c camera] bvh_file";
    public static void main(String[] args) throws ModelException, IOException, ParseException
    {
	// Get the first argument and initialize the factory
	DriverFactory factory=new EulerAnglesDriverFactory();
	String cameraType = null;
	String file=null;
	
	if(args.length < 1 || args.length > 5) {
	    throw new IllegalArgumentException(usage);
	} else {
	    for(int i = 0; i < args.length; i++) {
		if(args[i].equals("-i")) {
		    if (args[++i].equalsIgnoreCase("euler")) {
			factory=new EulerAnglesDriverFactory();
		    }
		    else if (args[i].equalsIgnoreCase("quartenion")) {
			factory=new QuartenionDriverFactory();
		    } 
		    else if (args[i].equalsIgnoreCase("map")) {
			factory=new MapDriverFactory();
		    }
		    else {
			throw new IllegalArgumentException("Unknown representation "+args[i]);
		    }		    
		} else if(args[i].equals("-c")) {
		    if (args[++i].equalsIgnoreCase("choppy")) {
			cameraType = "choppy";
		    } else {
			throw new IllegalArgumentException("Unknown camera "+args[i]);
		    }
		} else {
		    file=args[i];
		}
	    }
	}
	
	// Initialize a model
	LODBVHParser parser=new LODBVHParser(new FileInputStream(file));
	parser.init();
	ModelTimer timer=new ModelTimer(300);

	BasicAnimationModel model1=new BasicAnimationModel(timer);
	model1.setDriverFactory(factory);
	Map elevationMap=new HashMap();
	parser.fill(model1,elevationMap);
	parser.Hierarchy();
	
	//ViewFrame frame1=new ViewFrame(model1,new TreeModelView());

//  	// Add a new skeleton
//  	SkeletonHandle handle=model.addSkeleton(model.getRootJoint());
//  	handle.getDataManager().copyDriversFrom(model.getDataManager());
//  	handle.getDataManager().addTranslationDriver(0,model.getNumberOfFrames()-1,new DummyTranslationDriver(handle.getDataManager().getTranslationDriverFor(0,model.getNumberOfFrames()-1)));
	
	// Create the frame

	BasicAnimationModel model2=new BasicAnimationModel(new ModelTimer(300));
	model2.setDriverFactory(model1.getDriverFactory());
	model2.setNumberOfFrames(model1.getNumberOfFrames());
	SkeletonHandle shadow=model2.addSkeleton(parser.getRootHandle().getRootJoint());
	shadow.getDataManager().copyDriversFrom(parser.getRootHandle().getDataManager());
	shadow.getDataManager().copyOffsetsFrom(parser.getRootHandle().getDataManager());
	shadow.getDataManager().getRotationMatrix(shadow.getRootJoint());
	
	ViewFrame frame1=new ViewFrame(model2,new GLModelView(cameraType));
	//ViewFrame frame1=new ViewFrame(model2,new TreeModelView());
	WindowAdapter myWindowAdapter=new MyWindowAdapter();
	
	// frame2.addWindowListener(myWindowAdapter);
	frame1.addWindowListener(myWindowAdapter);
    }
}

class MyWindowAdapter extends WindowAdapter
{
    public void windowClosing(WindowEvent e) 
    {
	System.exit(0);
    }
}
