package mc.construction;

import mc.model.MappedTranslationDriver;
import mc.model.TranslationDriver;
import mc.model.Joint;
import mc.model.DataRepresentation;
import mc.model.TranslationVector3d;

import com.sun.java.util.collections.*;
import javax.vecmath.Matrix4d;

/**
 * A driver for offsets.
 */
public class OffsetDriver extends MappedTranslationDriver
{
    /** The map that holds the offsets */
    private final Map offsetMap;

    /**
     * Initailize.
     *
     * @param nativeDriver The native driver.
     * @param map The map
     */
    public OffsetDriver(TranslationDriver nativeDriver,Map map)
    {
	super(nativeDriver,map);
	this.offsetMap=new HashMap();
    }
    
    /**
     * Set the offset data for a joint.
     *
     * @param joint The joint.
     * @param matrix The matrix.
     */
    public void setOffset(Joint joint,Matrix4d matrix)
    {
	offsetMap.put(joint,matrix);
    }
    
    /**
     * Get the translation matrix.
     *
     * @param joint The joint.
     * @param frame The frame (not applicable here).
     */
    public Matrix4d getTranslationMatrix(Joint joint,int frame)
    {
	// Check the map
	Matrix4d result=(Matrix4d)offsetMap.get(joint);
	if (result==null) {
	    return super.getTranslationMatrix(joint,frame);
	} else {
	    return result;
	}
    }
    
    public DataRepresentation getTranslationRep(Joint joint,int frame)
    {
	// Check the map
	Matrix4d result=(Matrix4d)offsetMap.get(joint);
	if (result==null) {
	    return super.getTranslationRep(joint,frame);
	} else {
	    // Get the vector component
	    TranslationVector3d v=new TranslationVector3d(0,0,0);
	    result.get(v);
	    return v;
	}
    }
}

