/* Generated By:JavaCC: Do not edit this line. BVHParser.java */
package mc.io.bvh;

import mc.model.Joint;
import mc.model.BasicJoint;
import mc.model.SkeletonHandle;
import mc.model.BasicAnimationModel;
import mc.model.MutableRotationDriver;
import mc.model.MutableTranslationDriver;
import mc.model.DriverFactory;
import java.io.*;
import com.sun.java.util.collections.*;

/**
 * The parser for BVH files
 */
public class BVHParser implements BVHParserConstants {
    /**
     * The offset map.
     */
    private Vector offsets;

    /** The translation driver */
    private MutableTranslationDriver xlatDriver;

    /** The rot driver */
    private MutableRotationDriver rotDriver;

    /** The factory for drivers */
    private DriverFactory theFactory;

    /** The root joint of our model */
    private Joint theRootJoint;

    /** The frame count */
    private int frameCount;

    /** The index that is used whenever frame data are read */
    private int index;

    /**
     * An ordered list of data nodes for reading in motion data.
     */
    private Vector dataNodes;

    /**
     * The animation model that is being filled up.
     */
    private BasicAnimationModel animationModel;

    /**
     * Initialize the private variables of the parser.
     *
     */
    public void init()
    {
        this.dataNodes=new Vector();
        this.offsets=new Vector();
        this.theRootJoint=null;
        this.frameCount=0;
    }

    /** Instantiate the drivers */
    void setFactory(DriverFactory factory)
    {
        this.xlatDriver=factory.createMutableTranslationDriver();
        this.rotDriver=factory.createMutableRotationDriver();
        this.theFactory=factory;
    }

    /** Allocate the number of frames */
    private void allocateFrames(int frames)
    {
        // System.err.println("Allocating "+frames+" frames");
        animationModel.setNumberOfFrames(frames);
        xlatDriver.initialize(frames);
        rotDriver.initialize(frames);
    }


    /**
     * Set the offsets.
     *
     * @param handle The skeleton handle.
     */
    private void setOffsets(SkeletonHandle handle)
    {
        for(Iterator i=offsets.iterator();i.hasNext();) {
            OffsetData offset=(OffsetData)i.next();
            handle.getDataManager().setOffset(offset.joint,offset.x,offset.y,offset.z);
        }
        offsets.clear();
    }

    /**
     * Set the offset for a joint.
     *
     */
    private void setOffsetFor(BasicJoint joint,double x,double y,double z)
    {
        // System.err.println("Setting translation for "+joint.getName()+" to ["+x+","+y+","+z+"]");
        offsets.add(new OffsetData(joint,x,y,z));
    }

    private void initChannelsFor(Joint joint,int num)
    {
        // System.err.println("Allocating "+num+" channels for joint "+joint.getName());
    }

    private void addChannelFor(BasicJoint joint,String channelName)
    {
        // Allocate a data node
        DataNode node=new DataNode(joint,channelName);
        dataNodes.add(node);
    }

    private static final String ROTATEX="Xrotation";

    private static final String ROTATEY="Yrotation";

    private static final String ROTATEZ="Zrotation";

    private static final String TRANSLATEX="Xposition";

    private static final String TRANSLATEY="Yposition";

    private static final String TRANSLATEZ="Zposition";

    private double parseDouble(String s) throws NumberFormatException
    {
        return (new Double(s)).doubleValue();
    }

    private void recordValue(double value) throws ParseException
    {
        // Check if we should wrap around
        if (index==dataNodes.size()) {
            index=0;
            frameCount++;
        }

        // Index in our vector
        DataNode dataNode=(DataNode)dataNodes.elementAt(index);
        // Check the channel name
        if (dataNode.channelName.equals(TRANSLATEX)) {
            xlatDriver.setTranslateX(dataNode.joint,frameCount,value);
        }
        else if (dataNode.channelName.equals(TRANSLATEY)) {
            xlatDriver.setTranslateY(dataNode.joint,frameCount,value);
        }
        else if (dataNode.channelName.equals(TRANSLATEZ)) {
            xlatDriver.setTranslateZ(dataNode.joint,frameCount,value);
        }
        else if (dataNode.channelName.equals(ROTATEX)) {
            rotDriver.setRotateX(dataNode.joint,frameCount,Math.PI*value/180);
        }
        else if (dataNode.channelName.equals(ROTATEY)) {
            rotDriver.setRotateY(dataNode.joint,frameCount,Math.PI*value/180);
        }
        else if (dataNode.channelName.equals(ROTATEZ)) {
            rotDriver.setRotateZ(dataNode.joint,frameCount,Math.PI*value/180);
        } else {
            throw new ParseException("Cannot identify channel "+dataNode.channelName);
        }
        index++;
    }

    void fill(BasicAnimationModel animationModel)
    {
        this.animationModel=animationModel;
    }

    public static void main(String [] args)  throws ParseException,IOException
    {
        // Open the file
        BVHParser parser=new BVHParser(new FileInputStream(args[0]));
        parser.init();
        parser.Hierarchy();
    }

  final public void Hierarchy() throws ParseException {
    int frames;
    double time;
    Joint joint;
    Token x=null;
    jj_consume_token(HIERARCHY);
    jj_consume_token(ROOT);
    joint = Joint();

    jj_consume_token(MOTION);
    jj_consume_token(19);
    x = jj_consume_token(CONSTANT);
        // Parse the double
        try {
            frames=Integer.parseInt(x.image);
            allocateFrames(frames);
        }
        catch(NumberFormatException e) {
                // Ignore it
        }
    jj_consume_token(20);
    x = jj_consume_token(CONSTANT);
        // Parse the time
        try {
            time=parseDouble(x.image);
        }
        catch(NumberFormatException e) {
        }
    FrameData();
            // Set the root joint of the model
            animationModel.setDriverFactory(theFactory);
            SkeletonHandle handle=animationModel.addSkeleton(joint);
            setOffsets(handle);
            // Add the default strategy
            handle.getDataManager().addRotationDriver(0,animationModel.getNumberOfFrames()-1,rotDriver);
            handle.getDataManager().addTranslationDriver(0,animationModel.getNumberOfFrames()-1,xlatDriver);
  }

  final public BasicJoint Joint() throws ParseException {
    BasicJoint currentJoint=null;
    BasicJoint child=null;
    Token x=null;
    Token y=null;
    Token z=null;
    Token name;
    name = jj_consume_token(NAME);
    jj_consume_token(LBRACE);
        currentJoint=new BasicJoint(name.image);
    jj_consume_token(OFFSET);
    x = jj_consume_token(CONSTANT);
    y = jj_consume_token(CONSTANT);
    z = jj_consume_token(CONSTANT);
        try {
            setOffsetFor(currentJoint,parseDouble(x.image),parseDouble(y.image),parseDouble(z.image));
        }
        catch(NumberFormatException e) {
        }
    jj_consume_token(CHANNELS);
    z = jj_consume_token(CONSTANT);
        try {
            initChannelsFor(currentJoint,Integer.parseInt(z.image));
        }
        catch(NumberFormatException e) {
        }
    label_1:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case NAME:
        ;
        break;
      default:
        jj_la1[0] = jj_gen;
        break label_1;
      }
      x = jj_consume_token(NAME);
          addChannelFor(currentJoint,x.image);
    }
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case JOINT:
      label_2:
      while (true) {
        jj_consume_token(JOINT);
        child = Joint();
                currentJoint.addChild(child);
        switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
        case JOINT:
          ;
          break;
        default:
          jj_la1[1] = jj_gen;
          break label_2;
        }
      }
      break;
    case ENDSITE:
      jj_consume_token(ENDSITE);
      child = EndSite();
                currentJoint.addChild(child);
      break;
    default:
      jj_la1[2] = jj_gen;
      jj_consume_token(-1);
      throw new ParseException();
    }
    jj_consume_token(RBRACE);
        {if (true) return currentJoint;}
    throw new Error("Missing return statement in function");
  }

  final public BasicJoint EndSite() throws ParseException {
        BasicJoint currentJoint=null;
        Token x=null;
        Token y=null;
        Token z=null;
        Token name=null;
    jj_consume_token(LBRACE);
        currentJoint=new BasicJoint("End Site");
    jj_consume_token(OFFSET);
    x = jj_consume_token(CONSTANT);
    y = jj_consume_token(CONSTANT);
    z = jj_consume_token(CONSTANT);
        try {
            setOffsetFor(currentJoint,parseDouble(x.image),parseDouble(y.image),parseDouble(z.image));
        }
        catch(NumberFormatException e) {
        }
    jj_consume_token(RBRACE);
        {if (true) return currentJoint;}
    throw new Error("Missing return statement in function");
  }

  final public void FrameData() throws ParseException {
        frameCount=0;
        index=0;
        Token x=null;
    label_3:
    while (true) {
      x = jj_consume_token(CONSTANT);
                try {
                        recordValue(parseDouble(x.image));
                }
                catch(NumberFormatException e) {
                }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case CONSTANT:
        ;
        break;
      default:
        jj_la1[3] = jj_gen;
        break label_3;
      }
    }
  }

  public BVHParserTokenManager token_source;
  ASCII_CharStream jj_input_stream;
  public Token token, jj_nt;
  private int jj_ntk;
  private int jj_gen;
  final private int[] jj_la1 = new int[4];
  final private int[] jj_la1_0 = {0x1000,0x200,0xa00,0x8000,};

  public BVHParser(java.io.InputStream stream) {
    jj_input_stream = new ASCII_CharStream(stream, 1, 1);
    token_source = new BVHParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  public void ReInit(java.io.InputStream stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  public BVHParser(java.io.Reader stream) {
    jj_input_stream = new ASCII_CharStream(stream, 1, 1);
    token_source = new BVHParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  public void ReInit(java.io.Reader stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  public BVHParser(BVHParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  public void ReInit(BVHParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 4; i++) jj_la1[i] = -1;
  }

  final private Token jj_consume_token(int kind) throws ParseException {
    Token oldToken;
    if ((oldToken = token).next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    if (token.kind == kind) {
      jj_gen++;
      return token;
    }
    token = oldToken;
    jj_kind = kind;
    throw generateParseException();
  }

  final public Token getNextToken() {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    jj_gen++;
    return token;
  }

  final public Token getToken(int index) {
    Token t = token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  final private int jj_ntk() {
    if ((jj_nt=token.next) == null)
      return (jj_ntk = (token.next=token_source.getNextToken()).kind);
    else
      return (jj_ntk = jj_nt.kind);
  }

  private java.util.Vector jj_expentries = new java.util.Vector();
  private int[] jj_expentry;
  private int jj_kind = -1;

  final public ParseException generateParseException() {
    jj_expentries.removeAllElements();
    boolean[] la1tokens = new boolean[21];
    for (int i = 0; i < 21; i++) {
      la1tokens[i] = false;
    }
    if (jj_kind >= 0) {
      la1tokens[jj_kind] = true;
      jj_kind = -1;
    }
    for (int i = 0; i < 4; i++) {
      if (jj_la1[i] == jj_gen) {
        for (int j = 0; j < 32; j++) {
          if ((jj_la1_0[i] & (1<<j)) != 0) {
            la1tokens[j] = true;
          }
        }
      }
    }
    for (int i = 0; i < 21; i++) {
      if (la1tokens[i]) {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.addElement(jj_expentry);
      }
    }
    int[][] exptokseq = new int[jj_expentries.size()][];
    for (int i = 0; i < jj_expentries.size(); i++) {
      exptokseq[i] = (int[])jj_expentries.elementAt(i);
    }
    return new ParseException(token, exptokseq, tokenImage);
  }

  final public void enable_tracing() {
  }

  final public void disable_tracing() {
  }

}

class OffsetData
{
    public final double x;

    public final double y;

    public final double z;

    public final Joint joint;

    public OffsetData(Joint joint,double x,double y,double z)
    {
        this.joint=joint;
        this.x=x;
        this.y=y;
        this.z=z;
    }
}
