/* Generated By:JavaCC: Do not edit this line. LODBVHParser.java */
package mc.io.bvh;

import mc.model.Joint;
import mc.model.BasicJoint;
import mc.model.SkeletonHandle;
import mc.model.BasicAnimationModel;
import mc.model.MutableRotationDriver;
import mc.model.MutableTranslationDriver;
import mc.model.MappedRotationDriver;
import mc.model.TranslationDriver;
import mc.model.MappedTranslationDriver;
import mc.model.DriverFactory;
import mc.model.euler.EulerAnglesDriverFactory;
import mc.model.ModelTimer;
import mc.model.LODPyramid;
import mc.model.DataManager;
import mc.model.ElevationLODChooser;
import mc.model.DefaultLODChooser;

import java.io.*;
import com.sun.java.util.collections.*;

/**
 * The parser for BVH files
 */
public class LODBVHParser implements LODBVHParserConstants {
    /** The root rotation driver */
    private MutableRotationDriver rootRotDriver;

    /** The root translation driver */
    private MutableTranslationDriver rootXlatDriver;

    /** The map from root joints to skeletons */
    private Map skeletonMap;

    /** The elevation map */
    private Map elevationMap;

    /**
     * The offset map.
     */
    private Vector offsets;

    /** The factory for drivers */
    private DriverFactory theFactory;

    /** The handle */
    private SkeletonHandle theRootHandle;

    /** The main skeleton that is recorded in the animation model */
    private SkeletonHandle mainSkeleton;

    /** The LOD pyramid */
    private LODPyramid pyramid;

    /** The frame count */
    private int frameCount;

    /** The index that is used whenever frame data are read */
    private int index;

    /** The map that holds the ids for the root skeleton */
    private Map rootMap;

    /**
     * An ordered list of data nodes for reading in motion data.
     */
    private Vector dataNodes;

    /**
     * The map from joints to skeleton.
     */
    private Map jointToSkeleton;

    /** The map that holds info about the drivers that will
     * be pushed */
    private Map mappedDrivers;

    /**
     * The animation model that is being filled up.
     */
    private BasicAnimationModel animationModel;

    /**
     * Initialize the private variables of the parser.
     *
     */
    public void init()
    {
        this.dataNodes=new Vector();
        this.offsets=new Vector();
        this.theRootHandle=null;
        this.frameCount=0;
        this.skeletonMap=new HashMap();
        this.rootMap=new HashMap();
        this.mappedDrivers=new HashMap();
        this.jointToSkeleton=new HashMap();
    }

    /** Allocate the number of frames */
    private void allocateFrames(int frames)
    {
        // System.err.println("Allocating "+frames+" frames");
        animationModel.setNumberOfFrames(frames);

        // Add drivers for the root skeleton
        rootXlatDriver=animationModel.getDriverFactory().createMutableTranslationDriver();
        rootXlatDriver.initialize(frames);
        theRootHandle.getDataManager().addTranslationDriver(0,animationModel.getNumberOfFrames(),rootXlatDriver);
        // Add a rotation driver
        rootRotDriver=animationModel.getDriverFactory().createMutableRotationDriver();
        rootRotDriver.initialize(frames);
        theRootHandle.getDataManager().addRotationDriver(0,animationModel.getNumberOfFrames(),rootRotDriver);
    }

    private void allocateCloneFrames(int frames)
    {
        // Add the mapped drivers for everybody else
        for(Iterator hi=mappedDrivers.keySet().iterator();hi.hasNext();) {
            SkeletonHandle handle=(SkeletonHandle)hi.next();
            // Skip over the default level
            if (handle==theRootHandle)
                continue;

            Map jointMap=(Map)mappedDrivers.get(handle);
            handle.getDataManager().addRotationDriver(0,frames,new MappedRotationDriver(rootRotDriver,jointMap));
            TranslationDriver mappedDriver=new MappedTranslationDriver(rootXlatDriver,jointMap);
            handle.getDataManager().addTranslationDriver(0,frames,mappedDriver);
        }
    }

    private void buildMappings()
    {
        // Build the root mapping
        buildRootMapping(theRootHandle.getRootJoint());
        // Build a mapping for each of the other skeletons
        for(Iterator i=pyramid.getLevels();i.hasNext();) {
            SkeletonHandle handle=(SkeletonHandle)i.next();
            Map jointMap=buildMapping(handle);
            mappedDrivers.put(handle,jointMap);
        }
    }

    private void setOffsets(SkeletonHandle handle)
    {
        for(Iterator i=offsets.iterator();i.hasNext();) {
            LODOffsetData offset=(LODOffsetData)i.next();
            handle.getDataManager().setOffset(offset.joint,offset.x,offset.y,offset.z);
        }
    }

    /**
     * Set the offset for a joint.
     *
     */
    private void setOffsetFor(BasicJoint joint,double x,double y,double z)
    {
        // System.err.println("Setting translation for "+joint.getName()+" to ["+x+","+y+","+z+"]");
        offsets.add(new LODOffsetData(joint,x,y,z));
    }

    private void initChannelsFor(Joint joint,int num)
    {
        // System.err.println("Allocating "+num+" channels for joint "+joint.getName());
    }

    private void addChannelFor(BasicJoint joint,String channelName)
    {
        // Allocate a data node
        DataNode node=new DataNode(joint,channelName);
        dataNodes.add(node);
    }

    private static final String ROTATEX="Xrotation";

    private static final String ROTATEY="Yrotation";

    private static final String ROTATEZ="Zrotation";

    private static final String TRANSLATEX="Xposition";

    private static final String TRANSLATEY="Yposition";

    private static final String TRANSLATEZ="Zposition";

    private static final String ID="Id";

    private static final String ELEVATION="Elevation";

    private double parseDouble(String s) throws NumberFormatException
    {
        return (new Double(s)).doubleValue();
    }

    private int parseInt(String s) throws NumberFormatException
    {
        return (new Integer(s)).intValue();
    }

    private void recordValue(String valueStr) throws ParseException
    {
        try {
            // Check if we should wrap around
            if (index==dataNodes.size()) {
                index=0;
                frameCount++;
            }

            // Index in our vector
            DataNode dataNode=(DataNode)dataNodes.elementAt(index);
            // Get the skeleton handle
            SkeletonHandle handle=(SkeletonHandle)jointToSkeleton.get(dataNode.joint);
            // Get the drivers
            MutableRotationDriver rotDriver=(MutableRotationDriver)handle.getDataManager().getRotationDriverFor(0,1);
            MutableTranslationDriver xlatDriver=(MutableTranslationDriver)handle.getDataManager().getTranslationDriverFor(0,1);
            // Check the channel name
            if (dataNode.channelName.equals(TRANSLATEX)) {
                xlatDriver.setTranslateX(dataNode.joint,frameCount,parseDouble(valueStr));
            }
            else if (dataNode.channelName.equals(TRANSLATEY)) {
                xlatDriver.setTranslateY(dataNode.joint,frameCount,parseDouble(valueStr));
            }
            else if (dataNode.channelName.equals(TRANSLATEZ)) {
                xlatDriver.setTranslateZ(dataNode.joint,frameCount,parseDouble(valueStr));
            }
            else if (dataNode.channelName.equals(ROTATEX)) {
                rotDriver.setRotateX(dataNode.joint,frameCount,Math.PI*parseDouble(valueStr)/180);
            }
            else if (dataNode.channelName.equals(ROTATEY)) {
                rotDriver.setRotateY(dataNode.joint,frameCount,Math.PI*parseDouble(valueStr)/180);
            }
            else if (dataNode.channelName.equals(ROTATEZ)) {
                rotDriver.setRotateZ(dataNode.joint,frameCount,Math.PI*parseDouble(valueStr)/180);
            }
            else if (dataNode.channelName.equals(ID)) {
                dataNode.joint.setId(parseInt(valueStr));
            }
            else if (dataNode.channelName.equals(ELEVATION)) {
                elevationMap.put(skeletonMap.get(dataNode.joint),new Double(valueStr));
            } else {
                throw new ParseException("Cannot identify channel "+dataNode.channelName);
            }
            index++;
        }
        catch(NumberFormatException e) {
            throw new ParseException("Cannot parse number :"+valueStr);
        }
    }

    private void buildRootMapping(Joint joint)
    {
        rootMap.put(new Integer(joint.getId()),joint);
        for(Iterator i=joint.getChildren().iterator();i.hasNext();) {
            Joint childJoint=(Joint)i.next();
            buildRootMapping(childJoint);
        }
    }

    private Map buildMapping(SkeletonHandle handle)
    {
        Map result=new HashMap();

        // Get the root joint of the handle
        Joint rootJoint=handle.getRootJoint();
        // Iterate over the joints
        buildMapping(rootJoint,handle,result);
        return result;
    }

    /**
     * Create a mapping from joints to skeletons.
     *
     * @param handle The handle.
     * @param joint The joint.
     */
    private void invertMapping(SkeletonHandle handle,Joint joint)
    {
        jointToSkeleton.put(joint,handle);
        // Iterate over the children and recurse
        for(Iterator i=joint.getChildren().iterator();i.hasNext();) {
            Joint childJoint=(Joint)i.next();
            invertMapping(handle,childJoint);
        }
    }

    /**
     * Build the mapping between the joints in a LOD skeleton
     * and the root skeleton.
     *
     * @param joint The joint.
     * @param handle The LOD skeleton.
     * @param result The map that will contain the mapping.
     */
    private void buildMapping(Joint joint,SkeletonHandle handle,Map result)
    {
        // Index in the root map
        Joint corrJoint=(Joint)rootMap.get(new Integer(joint.getId()));
        if (corrJoint==null) {
            throw new Error("Cannot locate corresponding root skeleton joint for:"+joint.getId());
        }
        result.put(joint,corrJoint);
        for(Iterator i=joint.getChildren().iterator();i.hasNext();) {
            Joint childJoint=(Joint)i.next();
            buildMapping(childJoint,handle,result);
        }
    }

    public SkeletonHandle getRootHandle()
    {
        return theRootHandle;
    }


    /**
     * Specify which model and elevation map will be filled.
     *
     * @param animationModel The animation model.
     * @param elevationMap The elevation map.
     */
    public void fill(BasicAnimationModel animationModel,Map elevationMap)
    {
        this.animationModel=animationModel;
        this.elevationMap=elevationMap;
    }

    public static void main(String [] args)  throws ParseException,IOException
    {
        // Open the file
        LODBVHParser parser=new LODBVHParser(new FileInputStream(args[0]));
        parser.init();
        DriverFactory factory=new EulerAnglesDriverFactory();
        ModelTimer timer=new ModelTimer(300);
        BasicAnimationModel model=new BasicAnimationModel(timer);
        model.setDriverFactory(factory);
        Map elevationMap=new HashMap();
        parser.fill(model,elevationMap);
        parser.Hierarchy();
    }

  final public void Hierarchy() throws ParseException {
    int frames;
    double time;
    Joint joint;
    Token x=null;
    SkeletonHandle handle;
    jj_consume_token(HIERARCHY);
    label_1:
    while (true) {
      jj_consume_token(ROOT);
      joint = Joint();
                     handle=animationModel.createSkeleton(joint);
                     skeletonMap.put(joint,handle);

                     if (theRootHandle==null) {
                         mainSkeleton=animationModel.addSkeleton(handle);
                         pyramid=new LODPyramid(new DefaultLODChooser());
                         mainSkeleton.setLODPyramid(pyramid);
                         theRootHandle=handle;
                         buildRootMapping(joint);
                     } else {
                         pyramid.addLevel(handle);
                     }

                     invertMapping(handle,handle.getRootJoint());
                     setOffsets(handle);
                     //System.err.println("Recognized one skeleton");

      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case ROOT:
        ;
        break;
      default:
        jj_la1[0] = jj_gen;
        break label_1;
      }
    }
    jj_consume_token(MOTION);
    jj_consume_token(19);
    x = jj_consume_token(CONSTANT);
        // Parse the double
        try {
            frames=Integer.parseInt(x.image);
            allocateFrames(frames);
        }
        catch(NumberFormatException e) {
                // Ignore it
        }
    jj_consume_token(20);
    x = jj_consume_token(CONSTANT);
        // Parse the time
        try {
            time=parseDouble(x.image);
        }
        catch(NumberFormatException e) {
        }
    FrameData();
            // Build the mappings for the LOD skeletons
            buildMappings();
            // Allocate the drivers for the cloned skeletons
            allocateCloneFrames(animationModel.getNumberOfFrames());
            // Clone the driver in the main skeleton
            mainSkeleton.getDataManager().copyDriversFrom(theRootHandle.getDataManager());
            mainSkeleton.getDataManager().copyOffsetsFrom(theRootHandle.getDataManager());
            pyramid.addDefaultLevel(theRootHandle);
  }

  final public BasicJoint Joint() throws ParseException {
    BasicJoint currentJoint=null;
    BasicJoint child=null;
    Token x=null;
    Token y=null;
    Token z=null;
    Token name;
    name = jj_consume_token(NAME);
    jj_consume_token(LBRACE);
        currentJoint=new BasicJoint(name.image);
    jj_consume_token(OFFSET);
    x = jj_consume_token(CONSTANT);
    y = jj_consume_token(CONSTANT);
    z = jj_consume_token(CONSTANT);
        try {
            setOffsetFor(currentJoint,parseDouble(x.image),parseDouble(y.image),parseDouble(z.image));
        }
        catch(NumberFormatException e) {
        }
    Channels(currentJoint);
    label_2:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case JOINT:
        jj_consume_token(JOINT);
        child = Joint();
                currentJoint.addChild(child);
        break;
      case ENDSITE:
        jj_consume_token(ENDSITE);
        child = EndSite();
                currentJoint.addChild(child);
        break;
      default:
        jj_la1[1] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case JOINT:
      case ENDSITE:
        ;
        break;
      default:
        jj_la1[2] = jj_gen;
        break label_2;
      }
    }
    jj_consume_token(RBRACE);
        {if (true) return currentJoint;}
    throw new Error("Missing return statement in function");
  }

  final public BasicJoint EndSite() throws ParseException {
        BasicJoint currentJoint=null;
        Token x=null;
        Token y=null;
        Token z=null;
        Token name=null;
    jj_consume_token(LBRACE);
        currentJoint=new BasicJoint("End Site");
    jj_consume_token(OFFSET);
    x = jj_consume_token(CONSTANT);
    y = jj_consume_token(CONSTANT);
    z = jj_consume_token(CONSTANT);
        try {
            setOffsetFor(currentJoint,parseDouble(x.image),parseDouble(y.image),parseDouble(z.image));
        }
        catch(NumberFormatException e) {
        }
    switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
    case CHANNELS:
      Channels(currentJoint);
      break;
    default:
      jj_la1[3] = jj_gen;
      ;
    }
    jj_consume_token(RBRACE);
        {if (true) return currentJoint;}
    throw new Error("Missing return statement in function");
  }

  final public void Channels(BasicJoint currentJoint) throws ParseException {
    Token z=null;
    Token x=null;
    jj_consume_token(CHANNELS);
    z = jj_consume_token(CONSTANT);
        try {
            initChannelsFor(currentJoint,Integer.parseInt(z.image));
        }
        catch(NumberFormatException e) {
        }
    label_3:
    while (true) {
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case NAME:
        ;
        break;
      default:
        jj_la1[4] = jj_gen;
        break label_3;
      }
      x = jj_consume_token(NAME);
         addChannelFor(currentJoint,x.image);
    }
  }

  final public void FrameData() throws ParseException {
        frameCount=0;
        index=0;
        Token x=null;
    label_4:
    while (true) {
      x = jj_consume_token(CONSTANT);
                try {
                        recordValue(x.image);
                }
                catch(NumberFormatException e) {
                }
      switch ((jj_ntk==-1)?jj_ntk():jj_ntk) {
      case CONSTANT:
        ;
        break;
      default:
        jj_la1[5] = jj_gen;
        break label_4;
      }
    }
  }

  public LODBVHParserTokenManager token_source;
  ASCII_CharStream jj_input_stream;
  public Token token, jj_nt;
  private int jj_ntk;
  private int jj_gen;
  final private int[] jj_la1 = new int[6];
  final private int[] jj_la1_0 = {0x40,0xa00,0xa00,0x400,0x1000,0x8000,};

  public LODBVHParser(java.io.InputStream stream) {
    jj_input_stream = new ASCII_CharStream(stream, 1, 1);
    token_source = new LODBVHParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 6; i++) jj_la1[i] = -1;
  }

  public void ReInit(java.io.InputStream stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 6; i++) jj_la1[i] = -1;
  }

  public LODBVHParser(java.io.Reader stream) {
    jj_input_stream = new ASCII_CharStream(stream, 1, 1);
    token_source = new LODBVHParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 6; i++) jj_la1[i] = -1;
  }

  public void ReInit(java.io.Reader stream) {
    jj_input_stream.ReInit(stream, 1, 1);
    token_source.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 6; i++) jj_la1[i] = -1;
  }

  public LODBVHParser(LODBVHParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 6; i++) jj_la1[i] = -1;
  }

  public void ReInit(LODBVHParserTokenManager tm) {
    token_source = tm;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 6; i++) jj_la1[i] = -1;
  }

  final private Token jj_consume_token(int kind) throws ParseException {
    Token oldToken;
    if ((oldToken = token).next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    if (token.kind == kind) {
      jj_gen++;
      return token;
    }
    token = oldToken;
    jj_kind = kind;
    throw generateParseException();
  }

  final public Token getNextToken() {
    if (token.next != null) token = token.next;
    else token = token.next = token_source.getNextToken();
    jj_ntk = -1;
    jj_gen++;
    return token;
  }

  final public Token getToken(int index) {
    Token t = token;
    for (int i = 0; i < index; i++) {
      if (t.next != null) t = t.next;
      else t = t.next = token_source.getNextToken();
    }
    return t;
  }

  final private int jj_ntk() {
    if ((jj_nt=token.next) == null)
      return (jj_ntk = (token.next=token_source.getNextToken()).kind);
    else
      return (jj_ntk = jj_nt.kind);
  }

  private java.util.Vector jj_expentries = new java.util.Vector();
  private int[] jj_expentry;
  private int jj_kind = -1;

  final public ParseException generateParseException() {
    jj_expentries.removeAllElements();
    boolean[] la1tokens = new boolean[21];
    for (int i = 0; i < 21; i++) {
      la1tokens[i] = false;
    }
    if (jj_kind >= 0) {
      la1tokens[jj_kind] = true;
      jj_kind = -1;
    }
    for (int i = 0; i < 6; i++) {
      if (jj_la1[i] == jj_gen) {
        for (int j = 0; j < 32; j++) {
          if ((jj_la1_0[i] & (1<<j)) != 0) {
            la1tokens[j] = true;
          }
        }
      }
    }
    for (int i = 0; i < 21; i++) {
      if (la1tokens[i]) {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.addElement(jj_expentry);
      }
    }
    int[][] exptokseq = new int[jj_expentries.size()][];
    for (int i = 0; i < jj_expentries.size(); i++) {
      exptokseq[i] = (int[])jj_expentries.elementAt(i);
    }
    return new ParseException(token, exptokseq, tokenImage);
  }

  final public void enable_tracing() {
  }

  final public void disable_tracing() {
  }

}

class LODOffsetData
{
    public final double x;

    public final double y;

    public final double z;

    public final BasicJoint joint;

    public LODOffsetData(BasicJoint joint,double x,double y,double z)
    {
        this.joint=joint;
        this.x=x;
        this.y=y;
        this.z=z;
    }
}
