package mc.model;

//import java.util.*;
import com.sun.java.util.collections.*;

/**
 * The basic implementation of a joint
 */
public class BasicJoint implements Joint
{
    /** The name of the joint */
    private final String name;
    
    /** The children of the joint */
    private final Vector children;
    
    /** The parent of this joint */
    private Joint parent;

    /** The id of the joint*/
    private int id;
    
    /** The counter for the id */
    private static int idCounter=0;
    
    /**
     * Initialize the joint.
     *
     * @param name The name of the joint.
     */
    public BasicJoint(String name) 
    {
	this.name=name;
	this.id=idCounter++;
	this.children=new Vector();
	this.parent=null;
    }
    
    /**
     * Copy ctor.
     *
     * @param joint The other joint.
     */
    public BasicJoint(Joint joint)
    {
	this.id=joint.getId();
	this.name=joint.getName();
	this.children=new Vector();
	this.parent=null;
    }
    
    public int getId()
    {
	return id;
    }
    
    /**
     * Set the id of this node.
     *
     * @param newId The new id.
     */
    public void setId(int newId)
    {
	this.id=newId;
    }
    
    public Vector getChildren()
    {
	return children;
    }
    
    public Joint getParent()
    {
	return parent;
    }

    public String getName()
    {
	return name;
    }
    
    public void addChild(BasicJoint j)
    {
	// Remove the joint from its parent
	if (j.getParent()!=null) {
	    ((BasicJoint)j.getParent()).remove(j);
	}

	children.add(j);
	j.setParent(this);
    }

    public void insertChild(BasicJoint joint,int index)
    {
	// Remove the joint from its parent
	if (joint.getParent()!=null) {
	    ((BasicJoint)joint.getParent()).remove(joint);
	}
	
	children.insertElementAt(joint,index);
	joint.setParent(this);
    }
    
    /**
     * Get the child at a given position
     *
     * @param index The index.
     * @return The child.
     */
    public BasicJoint getChildAt(int index)
    {
	return (BasicJoint)children.get(index);
    }
    
    /**
     * Get the index of a child.
     *
     * @param child The child.
     * @return The index of the child.
     */
    public int getIndexOf(Joint child)
    {
	return children.indexOf(child);
    }
    
    /**
     * Get the children count.
     *
     * @return The children count.
     */
    public int getChildCount()
    {
	return children.size();
    }
    
    public void remove(BasicJoint joint)
    {
	children.remove(joint);
	joint.setParent(null);
    }
    
    void setParent(Joint parent)
    {
	this.parent=parent;
    }
}

    
	
