package mc.model;

import javax.vecmath.Matrix4d;

/**
 * A default translation driver that always returns the same matrix
 */
public class DefaultTranslationDriver implements TranslationDriver
{
    /** The identity matrix */
    private final static Matrix4d Identity;

    /** The matrix to return for the root */
    private final Matrix4d rootMatrix;
    
    static
    {
	Identity=new Matrix4d();
	Identity.setIdentity();
    }
    
    public DefaultTranslationDriver()
    {
	this.rootMatrix=Identity;
    }
    
    public DefaultTranslationDriver(Matrix4d rootMatrix)
    {
	this.rootMatrix=new Matrix4d(rootMatrix);
    }

    public Matrix4d getTranslationMatrix(Joint joint,int frame)
    {
	if (joint.getParent()!=null) {
	    return Identity;
	} else {
	    return rootMatrix;
	}
    }
    
    public DataRepresentation getTranslationRep(Joint joint,int frame)
    {
	return new TranslationVector3d(0,0,0);
    }
}

