package mc.model;

/**
 * Factory that instantiates drivers and interpolators.
 * 
 */
public interface DriverFactory 
{
    /**
     * Create a mutable rotation driver.
     *
     * @return A rotation driver.
     */
    public MutableRotationDriver createMutableRotationDriver();

    /**
     * Create a mutable xlat driver.
     *
     * @return A translation driver. 
     */
    public MutableTranslationDriver createMutableTranslationDriver();
    
    /**
     * Create an interpolator for rotations.
     *
     * @param description The description of the interpolator. This will be one of
     * the strings that the getInterpolatorDescriptions method returns.
     * @return The rotation interpolator
     */
    public RotationInterpolator createRotationInterpolator(String description);
    
    /**
     * Get the description strings of the interpolators that this
     * factory supports.
     *
     * @return An array of descriptions
     */
    public String[] getInterpolatorDescriptions();
    
    /**
     * Create an interpolator for translations.
     *
     * @return The translation interpolator.
     */
    public TranslationInterpolator createTranslationInterpolator();
    
    /**
     * Get the encoding of rotations.
     *
     * @return A string that describes the encoding.
     */
    public String getRotationEncoding();
    
    /**
     * Get the encoding for translations
     *
     * @return A string that describes the encoding.
     */
    public String getTranslationEncoding();
}

