package mc.model;

import mc.view.Camera;
import mc.util.Pair;
import mc.util.SceneUtil;
import com.sun.java.util.collections.*;

import javax.vecmath.Matrix4d;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

/**
 * Interface for choosing the appropriate level of detail for
 * a skeleton.
 */
public class ElevationLODChooser implements LODChooser
{
    /** The list of elevations */
    private final List elevations;
    
    /** The referring distance */
    private double referringDistance;
    
    public ElevationLODChooser(Map elevationMap,double referringDistance)
    {
	this.referringDistance=referringDistance;
	
	// Init the list of elevations
	elevations=new LinkedList();
	for(Iterator i=elevationMap.keySet().iterator();i.hasNext();) {
	    SkeletonHandle handle=(SkeletonHandle)i.next();
	    Object elevation=elevationMap.get(handle);
	    elevations.add(new Pair(handle,elevation));
	}
	// Sort this list
	Collections.sort(elevations,new ElevationComparator());
    }

    public SkeletonHandle choose(LODPyramid pyramid, Camera camera, int frame)
    {
	// Calculate the point of the root
	SkeletonHandle skeleton=pyramid.getDefaultLevel();
	Matrix4d rootMatrix=SceneUtil.findGlobalJointMatrix(skeleton,new HashMap(),skeleton.getRootJoint(),frame);
	Point3d origin=new Point3d();
	rootMatrix.transform(origin);
	// Get the camera point
	Vector3d offset=new Vector3d(origin);
	Vector3d eye=camera.getEye(frame);
	offset.sub(eye);
	double distance=offset.length()/referringDistance;
	System.err.println("Distance: "+distance);
	double prevDistance=0;
	// Search the list
	SkeletonHandle handle=null;
	for(Iterator i=elevations.iterator();i.hasNext();) {
	    Pair p=(Pair)i.next();
	    // Get the elevation
	    Double elevation=(Double)p.second;
	    System.err.println("Examining elevation "+elevation);
	    handle=(SkeletonHandle)p.first;	    
	    if (elevation.doubleValue()>distance) {
		break;
	    }
	}
	return handle;
    }
}

/**
 * The comparator class for sorting the elevation lsit.
 */
class ElevationComparator implements Comparator
{
    public int compare(Object x1,Object x2)
    {
	// Downcast
	Pair p1=(Pair)x1;
	Pair p2=(Pair)x2;
	// Compare the two elevations
	Double e1=(Double)p1.second;
	Double e2=(Double)p2.second;
	if (e1.doubleValue()-e2.doubleValue()>0) {
	    return 1;
	}
	else if (e1.doubleValue()-e2.doubleValue()<0) {
	    return -1;
	} else {
	    return 0;
	}
    }
}

