package mc.model;

import mc.view.Camera;
import com.sun.java.util.collections.Vector;
import com.sun.java.util.collections.Collection;
import com.sun.java.util.collections.Iterator;

/**
 * Collection of SkeletonHandles that are related to each other by
 * being LOD's of each other.
 */
public class LODPyramid
{
    /** collection of LODs */
    private Collection lods;
    /** Chooses which LOD to use */
    private LODChooser chooser;

    /** The default skeleton */
    private SkeletonHandle defaultSkeleton;
    
    /**
     * Empty set of LODs
     * @param the chooser for this pyramid
     */
    public LODPyramid(LODChooser chooser)
    {
	this.chooser = chooser;
	lods = new Vector();
    }

    /**
     * adds a level of detail to the pyramid of skeletons
     * @param skeleton
     */
    public void addLevel(SkeletonHandle skeleton)
    {
	lods.add(skeleton);
    }

    /**
     * Adds a level of detail to the pyramid of skeletons
     * and records the skeleton as the deafult level.
     *
     * @param handle The handle.
     */
    public void addDefaultLevel(SkeletonHandle handle)
    {
	this.defaultSkeleton=handle;
	addLevel(handle);
    }
    
    /**
     * Iterate over all LODs
     * @return the iterator on skeletons
     */
    public Iterator getLevels()
    {
	return lods.iterator();
    }

    /**
     * Set the chooser for this pyramid
     * @param LODChooser
     */
    public void setChooser(LODChooser chooser)
    {
	this.chooser=chooser;
    }

    /**
     * Get the skeleton for the default level (i.e. no elevation).
     *
     * @return The default skeleton
     */
    public SkeletonHandle getDefaultLevel()
    {
	return defaultSkeleton;
    }
    
    /**
     * Get the chooser for this pyramid
     * @return LODChooser
     */
    public LODChooser getChooser()
    {
	return chooser;
    }

    /**
     * Delegate the chooser of this pyramid to chooser an appropriate LOD
     * @param camera
     * @param frame
     * @see LODChooser
     */
    public SkeletonHandle choose(Camera camera, int frame)
    {
	if(chooser == null)
	    return null;
	return chooser.choose(this, camera, frame);
    }
}
