package mc.model;

import javax.vecmath.Matrix4d;
import com.sun.java.util.collections.Map;
import com.sun.java.util.collections.Iterator;

/**
 * A translation driver that stacks on top of another driver
 * and uses a mapping to get the matrices.
 */
public class MappedRotationDriver implements RotationDriver
{
    /** The real driver*/
    protected final RotationDriver theDriver;
    
    /** The map */
    protected final Map theMap;
    
    /**
     * Initialize.
     *
     * @param theDriver The Rotation driver.
     * @param theMap The joint map.
     */
    public MappedRotationDriver(RotationDriver theDriver,Map theMap)
    {
	this.theDriver=theDriver;
	this.theMap=theMap;
    }
    
        /**
     * Get the Rotation matrix of the node.
     *
     * @param joint The joint.
     * @param frameNumber The frame number
     * @return The homogeneous 4x4 matrix
     */
    public Matrix4d getRotationMatrix(Joint joint,int frameNumber)
    {
	// Look up the joint
	Joint mappedJoint=(Joint)theMap.get(joint);
	// Sanity check
	if (mappedJoint==null) {
	    System.err.println(joint);
	    for(Iterator i=theMap.keySet().iterator();i.hasNext();) {
		Joint key=(Joint)i.next();
		Joint value=(Joint)theMap.get(key);
		System.err.println("key:"+key+" ==> "+"value:"+value);
	    }
	    
	    throw new Error("Could not find mapping for joint "+joint.getId());
	}
	return theDriver.getRotationMatrix(mappedJoint,frameNumber);
    }

    /**
     * Get the internal representation of the Rotation.
     *
     * @return The internal representation.
     */
    public DataRepresentation getRotationRep(Joint joint,int frameNumber)
    {
	// Look up the joint
	Joint mappedJoint=(Joint)theMap.get(joint);
	// Sanity check
	if (mappedJoint==null) {
	    throw new Error("Could not find mapping for joint "+joint);
	}
	return theDriver.getRotationRep(mappedJoint,frameNumber);
    }

    public String getDescription()
    {
	return "Mapped";
    }
}


