package mc.model;

import java.util.EventObject;

/**
 * The event that describes a modification in an animation
 * model.
 */
public class ModelEvent extends EventObject
{
    
    /** The type of the current event */
    private int eventType;
    
    /**
     * The skeleton handle.
     */
    private SkeletonHandle handle;
    
    /**
     * Initialize variables.
     *
     * @param source The source of the event.
     * @param eventType The type of the event
     */
    ModelEvent(Object source,int eventType)
    {
	super(source);
	this.eventType=eventType;
    }
    
    public SkeletonHandle getHandle()
    {
	return handle;
    }
    
    /**
     * Initialize an event that takes a skeleton argument.
     *
     * @param source The source of the event.
     * @param type The event type.
     * @param handle The skeleton handle.
     */
    public ModelEvent(Object source,int type,SkeletonHandle handle)
    {
	this(source,type);
	this.handle=handle;
    }
    
    /** Get the type of the event */
    public int getType()
    {
	return eventType;
    }

    /** The constant for modification */
    public static final int MODEL_MODIFIED=0;
    
    /** The constant for stopping animation */
    public static final int ANIMATION_STOPPED=1;

    /** The constant for signalling a change in the loop */
    public static final int LOOP_CHANGED=2;

    /** A skeleton was changed */
    public static final int SKELETON_CHANGED=3;

    /** A skeleton has been added */
    public static final int SKELETON_ADDED=4;
}
