package mc.model;

import javax.swing.Timer;
import java.awt.event.*;
import com.sun.java.util.collections.*;

/**
 * The timer class. The general semantics here is that a timer sends a signal every
 * n milliseconds. If multiple signals are generated and are pending, then these are
 * coalesced in a single signal.
 * 
 * The current implementation acts as a wrapper around the swing Timer
 */
final public class ModelTimer implements ActionListener
{
    /** The timer */
    private Timer timer;

    /** The list of listeners */
    private List listeners;
    
    /**
     * Describe constructor here.
     *
     * @param delay The delay between successive firings
     * @param model the model that will be modified every time the timer expires
     */
    public ModelTimer(int delay)
    {
	this.timer=new Timer(delay,this);
	this.listeners=new LinkedList();
    }

    /** The increment */
    private int playIncrement;

    public final static int TICK=0;
    
    public final static int STOP=1;
    
    
    /**
     * Start the timer.
     *
     */
    public void start(int increment)
    {
	this.playIncrement=increment;
	timer.start();
    }
    
    /**
     * Fire a tick notificaiton to the listeners.
     *
     * @param e The action event.
     */
    public void actionPerformed(ActionEvent e)
    {
	fireNotification(TICK);
    }
    
    /**
     * Add an action listener.
     *
     * @param listener The listener to add.
     */
    public void addListener(TimerListener listener)
    {
	listeners.add(listener);
    }
    
    /**
     * Remove a listener.
     *
     * @param listener The listener to remove.
     */
    public void removeListener(TimerListener listener)
    {
	listeners.remove(listener);
    }
    
    /**
     * Stop the timer.
     *
     */
    public void stop()
    {
	timer.stop();
	fireNotification(STOP);
    }

    /**
     * Returns true if the timer is running.
     *
     * @return true/false
     */
    public boolean isRunning()
    {
	return timer.isRunning();
    }

    /**
     * Send a notification to our listeners.
     *
     * @param commandName The command name.
     */
    private void fireNotification(int type)
    {
	// Construct the event
	TimerEvent actionEvent=new TimerEvent(this,type,playIncrement);
	// Iterate over the listeners
	for(Iterator i=listeners.iterator();i.hasNext();) {
	    TimerListener l=(TimerListener)i.next();
	    // Pass the command on
	    l.timerFired(actionEvent);
	}
    }
}
