package mc.model;

/**
 * Retarget the motion of a source skeleton to that of a target
 * skeleton. The process will proceed with an IK solution of the
 * target's end effectors relative to the source followed by 
 * motion blending over time. 
 */
public interface MotionRetargeter
{
    /**
     * Retarget motion from source to target. The sideeffect will
     * transform the target DataSources to achieve the retargeting.
     * @param source skeleton
     * @param target skeleton
     */
    public void retarget(AnimationModel model,SkeletonHandle src, SkeletonHandle tgt);

    /**
     * Retarget motion from source to target. The sideeffect will
     * transform the target DataSources to achieve the retargeting at the given frame
     * @param source skeleton
     * @param target skeleton
     * @param frame
     */
    public void retarget(AnimationModel model,SkeletonHandle src, SkeletonHandle tgt, int frame);
}
