package mc.model;

/**
 * The interface for drivers that allow rotations to be set.
 */
public interface MutableRotationDriver extends RotationDriver
{
    /**
     * Set the rotation around the X axis for a joint at a specific frame.
     *
     * @param joint The joint.
     * @param frame The frame.
     * @param value The value.
     */
    public void setRotateX(Joint joint,int frame,double value);
    
    /**
     * Set the rotation around the X axis for a joint at a specific frame.
     *
     * @param joint The joint.
     * @param frame The frame.
     * @param value The value.
     */
    public void setRotateY(Joint joint,int frame,double value);

    /**
     * Set the rotation around the X axis for a joint at a specific frame.
     *
     * @param joint The joint.
     * @param frame The frame.
     * @param value The value.
     */
    public void setRotateZ(Joint joint,int frame,double value);

    /**
     * Initialize the driver for a number of frames.
     *
     * @param numberOfFrames The number of frames.
     */
    public void initialize(int numberOfFrames);
}
