package mc.model;

/**
 * Interface for drivers that can be modified.
 */
public interface MutableTranslationDriver extends TranslationDriver
{
    /**
     * Set the translate X channel.
     *
     * @param joint The joint.
     * @param frame The frame.
     * @param value The value.
     */
    public void setTranslateX(Joint joint,int frame,double value);
    
    /**
     * Set the translate Y channel for a joint and a specific frame.
     *
     *
     * @param joint The joint.
     * @param frame The frame.
     * @param value The value.
     */
    public void setTranslateY(Joint joint,int frame,double value);

    /**
     * Set the translate Z channel for a joint and a specific frame.
     *
     * @param joint The joint.
     * @param frame The frame.
     * @param value The value.
     */
    public void setTranslateZ(Joint joint,int frame,double value);

    /**
     * Initialize the driver for a number of frames.
     *
     * @param numberOfFrames The number of frames.
     */
    public void initialize(int numberOfFrames);
    
}
