package mc.model;

import com.sun.java.util.collections.*;
import javax.vecmath.Matrix4d;

/**
 * A driver for offsets.
 */
public class OffsetDriver implements TranslationDriver
{
    /** The map that holds the offsets */
    private final Map[] offsetMaps;

    /** The driver stacked beneath */
    private final TranslationDriver driver;
    
    /**
     * Initailize.
     *
     * @param nativeDriver The native driver.
     * @param map The map
     */
    public OffsetDriver(TranslationDriver driver,int frames)
    {
	this.driver=driver;
	this.offsetMaps=new Map[frames];
	for(int i=0;i<frames;i++) {
	    offsetMaps[i]=new HashMap();
	}
    }
    
    /**
     * Set the offset data for a joint.
     *
     * @param joint The joint.
     * @param matrix The matrix.
     */
    public void setOffset(Joint joint,Matrix4d matrix,int frame)
    {
	offsetMaps[frame].put(joint,matrix);
    }
    
    /**
     * Get the translation matrix.
     *
     * @param joint The joint.
     * @param frame The frame (not applicable here).
     */
    public Matrix4d getTranslationMatrix(Joint joint,int frame)
    {
	// Check the map
	Matrix4d original=driver.getTranslationMatrix(joint,frame);
	Matrix4d offset=(Matrix4d)offsetMaps[frame].get(joint);
	if (offset==null) {
	    offset=new Matrix4d();
	    offset.setIdentity();
	} 
	
	Matrix4d result=new Matrix4d();
	result.mul(original,offset);
	return result;
    }
    
    public DataRepresentation getTranslationRep(Joint joint,int frame)
    {
	Matrix4d matrix=getTranslationMatrix(joint,frame);
	TranslationVector3d xlat=new TranslationVector3d(0,0,0);
	matrix.get(xlat);
	return xlat;
    }
}

