package mc.model;

import mc.util.SceneUtil;
import mc.util.Pair;
import javax.vecmath.*;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.HashMap;

public class SimpleIKRetargeter implements MotionRetargeter
{
    private HashMap endEffectorMap;

    public SimpleIKRetargeter()
    {
	endEffectorMap = new HashMap();
    }

    public void retarget(AnimationModel model,SkeletonHandle src,SkeletonHandle target, int frame) {
	SkeletonHandle source=src.getLODPyramid().getDefaultLevel();
	remapEndEffectors(source, target);

	for(Iterator i = endEffectorMap.keySet().iterator(); i.hasNext(); ) {
	    // the end effector joints
	    Joint sj = (Joint)i.next();
	    Joint tj = (Joint)endEffectorMap.get(sj);
	    // the parent to child vectors
	    //System.out.println("found:" + findVectorHead(sj, source, target).getName());
	    //Vector3d sjv = createVector(source, sj, target, tj.getParent().getParent(), frame);
	    //Vector3d tjv = createVector(target, tj, target, tj.getParent().getParent(), frame);
	    Vector3d sjv = createVector(source, sj, target, findVectorHead(sj, source, target), frame);
	    Vector3d tjv = createVector(target, tj, target, findVectorHead(sj, source, target), frame);
	    // the IK solution
	    Pair solution = solveForGoal(sjv, tjv);
	    // the solution application
	    transformJoint(target, findVectorHead(sj, source, target), solution, 0, model.getNumberOfFrames()-1);
	}

    }

    public void retarget(AnimationModel model,SkeletonHandle src,SkeletonHandle target)
    {
	retarget(model, src, target, model.getCurrentFrame());
    }    

    private void remapEndEffectors(SkeletonHandle src, SkeletonHandle tgt)
    {
	HashMap srcEndEffectors = getEndEffectors(src, src.getRootJoint(), new HashMap());
	HashMap tgtEndEffectors = getEndEffectors(tgt, tgt.getRootJoint(), new HashMap());

	for(Iterator i = srcEndEffectors.keySet().iterator(); i.hasNext();) {
	    Object sjKey = i.next();
	    if(tgtEndEffectors.containsKey(sjKey)) {
		endEffectorMap.put(srcEndEffectors.get(sjKey), tgtEndEffectors.get(sjKey));
	    } else {
		handleUnmappedEndEffector((Joint)srcEndEffectors.get(sjKey), tgt);
	    }
	}
    }

    private HashMap getEndEffectors(SkeletonHandle handle, Joint joint, HashMap map)
    {
	if(joint.getChildren().size() == 0) {
	    map.put(new Integer(joint.getId()), joint);
	} 
	for(Iterator i = joint.getChildren().iterator(); i.hasNext();) {
	    getEndEffectors(handle, (Joint)i.next(), map);
	}
	return map;
    }

    private void handleUnmappedEndEffector(Joint joint, SkeletonHandle tgt)
    {
	// averaging?
    }

    private Joint findVectorHead(Joint srcEf, SkeletonHandle src, SkeletonHandle tgt)
    {
	Joint tgtEf = (Joint)endEffectorMap.get(srcEf);
	Joint srcJoint = srcEf.getParent();
	Joint tgtJoint = tgtEf.getParent();
	//System.out.println(srcJoint.getName() + "," + tgtJoint.getName());
	while(srcJoint != null && tgtJoint != null && srcJoint.getId() == tgtJoint.getId()) {
	    //System.out.println(srcJoint.getName() + "," + tgtJoint.getName());
	    srcJoint = srcJoint.getParent();
	    tgtJoint = tgtJoint.getParent();
	}
	// reaching the root implies no simplification took place along
	// this path up the tree
	if(tgtJoint == null) 
	    return tgtEf.getParent();
	return tgtJoint;
    }

    private Vector3d createVector(SkeletonHandle tail, Joint tailJoint, 
				  SkeletonHandle head, Joint headJoint, int frame)
    {
	//System.out.println("Vector between : " + headJoint.getName() + "," + tailJoint.getName());
	Matrix4d hm = SceneUtil.findGlobalJointMatrix(head, new HashMap(), headJoint, frame);
	Matrix4d tm = SceneUtil.findGlobalJointMatrix(tail, new HashMap(), tailJoint, frame);
	
	Point3d hp = new Point3d();
	hm.transform(hp);
	Point3d tp = new Point3d();
	tm.transform(tp);
	//System.out.println(hp + "," + tp);
	return new Vector3d(hp.x - tp.x,
			    hp.y - tp.y,
			    hp.z - tp.z);
    }

    private Pair solveForGoal(Vector3d goal, Vector3d start)
    {
	double theta = start.angle(goal);
	Vector3d normal = new Vector3d();
	goal.normalize();
	start.normalize();
	normal.cross(start, goal);
	normal.normalize();
	if(Double.isNaN(normal.x) || Double.isNaN(normal.y) || Double.isNaN(normal.z)){
	    return null;
	}
	return new Pair(new Double(theta), normal);
    }

    private void transformJoint(SkeletonHandle handle, Joint joint, Pair angles, 
				int startFrame, int endFrame)
    {
	Matrix4d matrix = new Matrix4d();
	AxisAngle4d axisAngle = null;
	if(angles != null) {
	    Vector3d axis = (Vector3d)angles.second;
	    axisAngle = new AxisAngle4d(axis.x,axis.y,axis.z,((Double)angles.first).doubleValue());
	}

	matrix.setIdentity();
	if(axisAngle != null)
	    matrix.setRotation(axisAngle);

	DataManager dm = handle.getDataManager();
	RotationDriver rd = dm.getRotationDriverFor(startFrame, endFrame);

	if(!(rd instanceof TransformRotationDriver)) {
	    dm.addRotationDriver(startFrame, endFrame, new TransformRotationDriver(rd));
	    rd = dm.getRotationDriverFor(startFrame, endFrame);
	}
	((TransformRotationDriver)rd).mapRotation(joint, matrix);
    }
}
