package mc.model;

import com.sun.java.util.collections.Comparable;


/**
 * The handle for skeletons.
 */
public final class SkeletonHandle implements Comparable
{
    /** The root joint */
    private Joint theRoot;
    
    /** The data manager */
    private DataManager theDataManager;

    /**
     * The id of the skeleton
     */
    private final int id;

    /** LOD pyramid for this model */
    private LODPyramid pyramid;

    /**
     * Initialize the skeleton.
     *
     * @param root The root joint.
     * @param dataManager The data manager.
     */
    public SkeletonHandle(Joint root,DataManager dataManager)
    {
	this.theRoot=root;
	this.theDataManager=dataManager;
	this.id=getUniqueId();
	this.pyramid = null;
    }

    /**
     * Compare this object to another handle. Comparison
     * is based on handles.
     *
     * @param o The other object.
     * @return -1,0,1
     */
    public int compareTo(Object o)
    {
	SkeletonHandle handle=(SkeletonHandle)o;
	return id-handle.id;
    }
    
    /** The unique id */
    private static int globalId=0;
    
    /**
     * Get the id of the skeleton.
     *
     * @return The id.
     */
    public int getId()
    {
	return id;
    }
    
    /**
     * Get a unique id
     *
     * @return A unique id.
     */
    private static int getUniqueId()
    {
	return globalId++;
    }
    
    /**
     * Check whether this handle is equal to
     * another.
     *
     * @param o The other object.
     * @return true/false
     */
    public boolean equals(Object o)
    {
	if (o==null) {
	    return false;
	}
	else if (!(o instanceof SkeletonHandle)) {
	    return false;
	} else {
	    SkeletonHandle handle=(SkeletonHandle)o;
	    return handle.id==id;
	}
    }
    
    /**
     * Get the root joint for this handle.
     *
     * @return The root joint.
     */
    public Joint getRootJoint()
    {
	return theRoot;
    }

    /**
     * Set the new root for the skeleton.
     *
     * @param newRoot The new root.
     */
    void setRootJoint(Joint newRoot)
    {
	this.theRoot=newRoot;
    }
    
    /**
     * Set the data manager for this skeleton.
     *
     * @param theDataManager the new data manager.
     */
    void setDataManager(DataManager theDataManager)
    {
	this.theDataManager=theDataManager;
    }
    
    /**
     * Get the data manager.
     *
     * @return The data manager.
     */
    public DataManager getDataManager()
    {
	return theDataManager;
    }

    /**
     * Get the LODPyramid
     * @return LODPyramid
     */
    public LODPyramid getLODPyramid()
    {
	return pyramid;
    }

    /**
     * Set the LODPyramid
     * @param LODPyramid
     */
    public void setLODPyramid(LODPyramid pyramid)
    {
	this.pyramid = pyramid;
    }
}
