package mc.model;

import mc.util.Pair;
import mc.model.euler.EulerRotationDriver;
import javax.vecmath.Matrix4d;
import javax.vecmath.Quat4d;
import com.sun.java.util.collections.HashMap;

/**
 * Add a translation to the lower driver
 */
public class TransformRotationDriver implements RotationDriver
{
    /**
     * The wrapped driver
     */
    private final RotationDriver realDriver;
    private HashMap map;
    
    public TransformRotationDriver(RotationDriver realDriver)
    {
	this.realDriver=realDriver;
	this.map = new HashMap();
    }
    
    public void mapRotation(Joint joint, Pair extra)
    {
	map.put(joint, extra);
    }

    public void mapRotation(Joint joint, Matrix4d extra)
    {
	map.put(joint, extra);
    }

    public Matrix4d getRotationMatrix(Joint joint,int frameNumber)
    {
	Matrix4d matrix=realDriver.getRotationMatrix(joint,frameNumber);
	Matrix4d temp = matrix;
	Matrix4d extra = (Matrix4d)map.get(joint);

	if (extra != null) {
	    temp = new Matrix4d(matrix);
	    temp.mul(extra);
	}
	return temp;
    }
    private double tod(double r)
    {
	return r*180/Math.PI;
    }
    public DataRepresentation getRotationRep(Joint joint,int frameNumber)
    {
	return realDriver.getRotationRep(joint,frameNumber);
    }

    public String getDescription()
    {
	return "Rotation Xform";
    }
}

