package mc.model.euler;

import mc.model.DriverFactory;
import mc.model.BasicTranslationDriver;
import mc.model.MutableTranslationDriver;
import mc.model.MutableRotationDriver;
import mc.model.RotationInterpolator;
import mc.model.TranslationInterpolator;
import mc.model.TranslationVector3d;

/** 
 * The factory that uses Euler angles for rotations.
 */
public class EulerAnglesDriverFactory implements DriverFactory
{
    private final static String[] descriptions={ 
	"Linear Interpolator","Median Filter"
    };
    
    public MutableTranslationDriver createMutableTranslationDriver()
    {
	return new BasicTranslationDriver();
    }

    public String[] getInterpolatorDescriptions()
    {
	return descriptions;
    }
    
    public MutableRotationDriver createMutableRotationDriver()
    {
	return new BasicEulerRotationDriver();
    }
    
    public RotationInterpolator createRotationInterpolator(String description)
    {
	if (description.equals("Linear Interpolator")) {
	    return new EulerRotationInterpolator();
	}
	else if (description.equals("Median Filter")) {
	    return new EulerMedianFilterDriver();
	} else {
	    return null;
	}
    }

    public TranslationInterpolator createTranslationInterpolator()
    {
	return null;
    }

    public String getRotationEncoding()
    {
	return EulerRotationVector3d.Name;
    }
    
    public String getTranslationEncoding()
    {
	return TranslationVector3d.Name;
    }
}
