package mc.model.euler;

import mc.model.MutableRotationDriver;
import mc.model.Joint;
import mc.model.FrameDataArray;
import mc.model.DataRepresentation;
import mc.model.RotationDriver;

import java.util.*;
import com.sun.java.util.collections.*;
import javax.vecmath.Vector3d;
import javax.vecmath.Matrix4d;

/**
 * The basic driver that uses precomputed data to drive
 * the whole process.
 */
public interface EulerRotationDriver extends RotationDriver
{
    /**
     * Get the rotateX channel of a node, for a given
     * frame number.
     *
     * @param joint The joint.
     * @param frameNumber The frame number.
     * @return The value of the channel.
     */
    public double getRotateX(Joint joint,int frameNumber);
    
   /**
     * Get the rotateY channel of a node, for a given
     * frame number.
     *
     * @param joint The joint.
     * @param frameNumber The frame number.
     * @return The value of the channel.
     */
    public double getRotateY(Joint joint,int frameNumber);

    /**
     * Get the rotateZ channel of a node, for a given
     * frame number.
     *
     * @param joint The joint.
     * @param frameNumber The frame number.
     * @return The value of the channel.
     */
    public double getRotateZ(Joint joint,int frameNumber);

    /** The string constant for the X axis */
    public static final int X=0;
    
    /** The string constant for the Y axis */
    public static final int Y=1;
    
    /** The string constant for the Z axis */
    public static final int Z=2;
    
    /**
     * Get the order in which the euler angles have been applied.
     *
     * @param joint The joint.
     * @return An array of the symbols
     */
    public int[] getRotationAxisOrder(Joint joint);
}
