package mc.model.map;

import mc.model.DriverFactory;
import mc.model.MutableRotationDriver;
import mc.model.RotationInterpolator;
import mc.model.MutableTranslationDriver;
import mc.model.TranslationInterpolator;
import mc.model.TranslationVector3d;
import mc.model.BasicTranslationDriver;

public class MapDriverFactory implements DriverFactory
{
    private final static String[] descriptions={
	"Linear Interpolator",
	"Aligned Interpolator"
    };
    
    public MutableRotationDriver createMutableRotationDriver()
    {
	return new BasicMapRotationDriver();
    }
    
    public RotationInterpolator createRotationInterpolator(String description)
    {
	if (description.equals("Linear Interpolator")) {
	    return new MapInterpolator();
	} else if (description.equals("Aligned Interpolator")) {
	    return new AlignedMapInterpolator();
	} else {
	    return null;
	}
    }

    public String[] getInterpolatorDescriptions()
    {
	return descriptions;
    }
    
    public MutableTranslationDriver createMutableTranslationDriver()
    {
	return new BasicTranslationDriver();
    }
    
    public TranslationInterpolator createTranslationInterpolator()
    {
	return null;
    }

    public String getRotationEncoding()
    {
	return ParameterizedQuat4d.Name;
    }
    
    public String getTranslationEncoding()
    {
	return TranslationVector3d.Name;
    }
}

