package mc.model.map;

import mc.model.RotationInterpolator;
import mc.model.DataRepresentation;
import mc.model.Joint;
import mc.model.RotationDriver;

import javax.vecmath.Matrix4d;


class MapInterpolator implements RotationInterpolator, MapRotationDriver
{
    /** The starting frame */
    private int startFrame;
    
    /** The end frame */
    private int endFrame;
    
    /** The wrapped driver */
    private MapRotationDriver theDriver;
    
    public MapInterpolator()
    {}

    public String getDescription()
    {
	return "Linear Interpolation";
    }
    
    public void interpolate(int startFrame,int endFrame,RotationDriver driver)
    {
	this.startFrame=startFrame;
	this.endFrame=endFrame;
	if (!(driver instanceof MapRotationDriver)) {
	    throw new IllegalArgumentException("Incompatible driver type "+driver);
	}
	this.theDriver=(MapRotationDriver)driver;
    }

    /**
     * Get the rotation matrix
     *
     * @param joint The joint.
     * @param frame The frame.
     * @return The matrix.
     */
    public Matrix4d getRotationMatrix(Joint joint,int frame)
    {
	// Get the parameterized quat
	ParameterizedQuat4d quat=getParameterizedQuat4d(joint,frame);
	return quat.getMatrix4d();
    }
    
    /**
     * Linearly interpolate to produce the parameterized quartenion.
     *
     * @param joint The joint.
     * @param frame The frame.
     * @return The parameterized quat.
     */
    public ParameterizedQuat4d getParameterizedQuat4d(Joint joint,int frame)
    {
	// Get the parameterized quat for the first frame
	ParameterizedQuat4d startQuat=theDriver.getParameterizedQuat4d(joint,startFrame);
	ParameterizedQuat4d endQuat=theDriver.getParameterizedQuat4d(joint,endFrame);
	// Interpolate
	ParameterizedQuat4d result=new ParameterizedQuat4d();
	result.interpolate(startQuat,endQuat,(frame-startFrame)/(double)(endFrame-startFrame));
	return result;
    }
    
    /**
     * Get the parameterized quartenion for a joint and the frame number
     *
     * @param joint The joint
     * @param frameNumber The frame number
     * @return The parameterized quartenion
     */
    public DataRepresentation getRotationRep(Joint joint,int frameNumber)
    {
	// return the parameterized quartenion
	return getParameterizedQuat4d(joint,frameNumber);
    }
}

