package mc.model.quart;

import mc.model.RotationInterpolator;
import mc.model.Joint;
import mc.model.DataRepresentation;
import mc.model.RotationDriver;

import javax.vecmath.Matrix4d;
import javax.vecmath.Quat4d;

/**
 * An interpolator for quartenions
 */
public class QuartenionInterpolator implements QuartenionRotationDriver, RotationInterpolator
{
    /** The start frame */
    private int startFrame;
    
    /** The end frame */
    private int endFrame;
    
    /** The driver */
    private QuartenionRotationDriver theDriver;
    
    public QuartenionInterpolator()
    {}

    public String getDescription()
    {
	return "Slerp Interpolation";
    }
    
    /**
     * Set the interpolation range.
     *
     * @param startFrame The start frame
     * @param endFrame The end frame
     * @param theDriver The driver that will provide the interpolation data.
     */
    public void interpolate(int startFrame,int endFrame,RotationDriver theDriver)
    {
	this.startFrame=startFrame;
	this.endFrame=endFrame;
	if (!(theDriver instanceof QuartenionRotationDriver)) {
	    throw new IllegalArgumentException("Cannot used non-quartenion driver "+theDriver);
	}
	
	this.theDriver=(QuartenionRotationDriver)theDriver;
    }
    
    public Quat4d getQuartenionFor(Joint joint,int frameNumber)
    {
	Quat4d start=theDriver.getQuartenionFor(joint,startFrame);
	Quat4d end=theDriver.getQuartenionFor(joint,endFrame);
	Quat4d result=new Quat4d();
	// Interpolate
	result.interpolate(start,end,(frameNumber-(double)startFrame)/(double)(endFrame-startFrame));
	return result;
    }

    public Matrix4d getRotationMatrix(Joint joint,int frameNumber)
    {
	Matrix4d result=new Matrix4d();
	result.set(getQuartenionFor(joint,frameNumber));
	return result;
    }

    public DataRepresentation getRotationRep(Joint joint,int frameNumber)
    {
	return new QuartenionRepresentation(getQuartenionFor(joint,frameNumber));
    }
}


