package mc.test;

import mc.io.bvh.LODBVHParser;
import mc.io.bvh.ParseException;
import mc.model.*;
import mc.view.ViewFrame;
import mc.view.TreeModelView;
import mc.view.GLModelView;
import mc.view.Camera;
import mc.view.SceneProperties;
import mc.view.BoundingVolume;
import mc.view.BasicShapeAdapter;
import mc.view.cameras.*;
import mc.view.shapes.*;
import mc.model.ModelException;
import mc.model.euler.EulerAnglesDriverFactory;
import mc.model.quart.QuartenionDriverFactory;
import mc.model.map.MapDriverFactory;
import mc.model.DriverFactory;
import mc.util.SceneUtil;
import mc.util.Pair;
import mc.util.SwingingCameraController;

import com.sun.java.util.collections.*;
import java.io.*;
import java.awt.event.*;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;
import javax.vecmath.Matrix4d;
import javax.vecmath.Color4f;
import java.awt.Color;
import javax.vecmath.Point3d;

public class ALL_IK_CLOSE
{
    public static final String usage = "Usage: TestLOD BVH_file";
    public static void main(String[] args) throws ModelException, IOException, ParseException, NumberFormatException
    {
	// Get the first argument and initialize the factory
	DriverFactory factory=new EulerAnglesDriverFactory();
	String cameraType = null;
	String file=null;
	
	if(args.length < 1) {
	    throw new IllegalArgumentException(usage);
	} else {
	    file=args[0];
	}
	
	Matrix4d offset=new Matrix4d();
	Vector3d lookAt=new Vector3d();
	if (args.length<4) {
	    offset.setIdentity();
	}else{
	    offset.set(new Vector3d(parseDouble(args[1]),parseDouble(args[2]),parseDouble(args[3])));
	    offset.get(lookAt);
	    offset.m23+=40;
	}
	
	double startingDistance=parseDouble(args[4]);
	double endingDistance=parseDouble(args[5]);
	double referringDistance=parseDouble(args[6]);
	int framesCycle=parseInt(args[7]);

	// Initialize a model
	LODBVHParser parser=new LODBVHParser(new FileInputStream(file));
	parser.init();
	ModelTimer timer=new ModelTimer(330);
	LODAnimationModel model=new LODAnimationModel(timer,null);
	model.setDriverFactory(factory);
	HashMap elevationMap=new HashMap();
	parser.fill(model,elevationMap);
	parser.Hierarchy();

	// Take care of the original skeleton
	Matrix4d displacement=new Matrix4d();
	displacement.set(new Vector3d(0,0,-40));
	SkeletonHandle skeleton=(SkeletonHandle)model.getHandles().iterator().next();
	skeleton.getLODPyramid().setChooser(new ConstantLODChooser(skeleton.getLODPyramid().getDefaultLevel()));
	model.setRetargeter(skeleton,new SimpleIKRetargeter());
	cancelOutTranslations(model.getNumberOfFrames(),skeleton,offset);

	// Create the first LOD 
	SkeletonHandle first=SceneUtil.cloneSkeletonSameJoints(model,skeleton);
	offset.mul(displacement);
	cancelOutTranslations(model.getNumberOfFrames(),first,offset);
	// Get the appropriate skeleton
	Iterator i=first.getLODPyramid().getLevels();
	first.getLODPyramid().setChooser(new ConstantLODChooser((SkeletonHandle)i.next()));
	model.setRetargeter(first,new SimpleIKRetargeter());
	
	// Create the second LOD
	SkeletonHandle second=SceneUtil.cloneSkeletonSameJoints(model,skeleton);
	offset.mul(displacement);
	cancelOutTranslations(model.getNumberOfFrames(),second,offset);
	// Get the appropriate skeleton
	i=second.getLODPyramid().getLevels();
	i.next();
	second.getLODPyramid().setChooser(new ConstantLODChooser((SkeletonHandle)i.next()));
	model.setRetargeter(second,new SimpleIKRetargeter());
	
	// Create the view
	GLModelView view = new GLModelView(cameraType);
	model.setMasterView(view);
	// setup some scene properties (no gl requirements)
	SceneProperties props = view.getSceneProperties();
	props.addLightInfo(new Vector3f(0.0f, 100.0f, 0.0f),
			   new Vector3f(0.5f, 0.5f, 0.5f),
			   new Vector3f(1.0f, 1.0f, 1.0f),
			   0);
	props.setMaterialInfo(new Vector3f(1.0f, 1.0f, 1.0f), 128);
	EllipsoidShape ellipsoid = new EllipsoidShape(50,50);
	ellipsoid.setColor(new Color4f(0.5273f, 0.8047f, 0.9766f, 0));
	for(i=model.getHandles().iterator();i.hasNext();) {
	    SkeletonHandle skeletonHandle=(SkeletonHandle)i.next();
	    props.setSkeletonSegmentToShape(skeletonHandle, ellipsoid);
	    props.setSkeletonFatness(skeletonHandle, 3.0);
	    // leave joint as default
	    // leave tracers as default
	    LODPyramid p = skeletonHandle.getLODPyramid();
	    if(p != null) {
		for(Iterator j = p.getLevels(); j.hasNext(); ) {
		    SkeletonHandle ss = (SkeletonHandle)j.next();
		    props.setSkeletonSegmentToShape(ss, ellipsoid);
		    props.setSkeletonFatness(ss, 3.0);
		}
	    }
	}

	ViewFrame frame=new ViewFrame(model,view);
	view.getCamera().setTargetX(lookAt.x);
	view.getCamera().setTargetY(lookAt.y);
	view.getCamera().setTargetZ(lookAt.z);
	view.setCameraDebug(false);
	WindowAdapter myWindowAdapter=new MyWindowAdapter();
	frame.addWindowListener(myWindowAdapter);	

	// properties that require gl to be setup
	Matrix4d localFloor = new Matrix4d();
	localFloor.setIdentity();
	Pair p = SceneUtil.getBoundingVolume(model, 1);
	Matrix4d globalFloor = new Matrix4d();
	globalFloor.setIdentity();
	globalFloor.set(new Vector3d(0,((Vector3d)p.second).y-((BoundingVolume)p.first).getHeight()/2, 0));
	props.addIndependentObject(new BasicShapeAdapter(new BoundingVolume(200, 200, 2),localFloor),
				   new CheckeredPlaneShape(new Color(238,220,130), 
							   new Color(255,174,185),
							   view.getGL()), 
				   new Color4f(0f,0f,0f,0f),
				   globalFloor);
    }

    private static void cancelOutTranslations(int frames,SkeletonHandle skeleton,Matrix4d offset)
    {
	skeleton.getDataManager().addTranslationDriver(0,frames,new DefaultTranslationDriver(offset));
	if (skeleton.getLODPyramid()==null)
	    return;
	
	for(Iterator i=skeleton.getLODPyramid().getLevels();i.hasNext();) {
	    SkeletonHandle handle=(SkeletonHandle)i.next();
	    handle.getDataManager().addTranslationDriver(0,frames,new DefaultTranslationDriver(offset));
	}
    }
    
    private static double calculateDistance(SkeletonHandle skeletonHandle,Vector3d v)
    {
	// Get the world matrix of the root joint
	Matrix4d rootMatrix=SceneUtil.findGlobalJointMatrix(skeletonHandle,new HashMap(),skeletonHandle.getRootJoint(),0);
	Point3d p=new Point3d(0,0,0);
	rootMatrix.transform(p);
	Vector3d pv=new Vector3d(p);
	pv.sub(v);
	return pv.length();
    }

    private static int parseInt(String str) throws NumberFormatException
    {
	return (new Integer(str)).intValue();
    }
    
    private static double parseDouble(String str) throws NumberFormatException
    {
	return (new Double(str)).doubleValue();
    }
    
}

class MyWindowAdapter extends WindowAdapter
{
    public void windowClosing(WindowEvent e) 
    {
	System.exit(0);
    }
}
