package mc.test;

import mc.io.bvh.LODBVHParser;
import mc.io.bvh.ParseException;
import mc.model.*;
import mc.view.ViewFrame;
import mc.view.TreeModelView;
import mc.view.GLModelView;
import mc.view.Camera;
import mc.view.cameras.*;
import mc.model.ModelException;
import mc.model.euler.EulerAnglesDriverFactory;
import mc.model.quart.QuartenionDriverFactory;
import mc.model.map.MapDriverFactory;
import mc.model.DriverFactory;
import mc.util.SceneUtil;

import com.sun.java.util.collections.*;
import java.io.*;
import java.awt.event.*;
import javax.vecmath.Vector3d;

public class TestLOD
{
    public static final String usage = "Usage: TestLOD BVH_file";
    public static void main(String[] args) throws ModelException, IOException, ParseException
    {
	// Get the first argument and initialize the factory
	DriverFactory factory=new EulerAnglesDriverFactory();
	String cameraType = null;
	String file=null;
	
	if(args.length < 1 || args.length > 5) {
	    throw new IllegalArgumentException(usage);
	} else {
	    file=args[0];
	}
	
	// Initialize a model
	LODBVHParser parser=new LODBVHParser(new FileInputStream(file));
	parser.init();
	ModelTimer timer=new ModelTimer(33);

	BasicAnimationModel model=new BasicAnimationModel(timer);
	model.setDriverFactory(factory);
	HashMap elevationMap=new HashMap();
	parser.fill(model,elevationMap);
	parser.Hierarchy();
	// Get the original skeleton
	SkeletonHandle original=model.getFirstSkeleton();
	// Clone with a translation
	SkeletonHandle displaced=SceneUtil.cloneSkeletonSameJoints(model,
								   original,
								   new Vector3d(new Double(args[1]).doubleValue(),
										new Double(args[2]).doubleValue(),
										new Double(args[3]).doubleValue()));
	// Get the pyramid and the first skeleton
	LODPyramid pyramid=displaced.getLODPyramid();
	SkeletonHandle lodSkeleton=(SkeletonHandle)pyramid.getLevels().next();
	// Clone it
	ModelTimer newTimer=new ModelTimer(33);
	LODAnimationModel testModel=new LODAnimationModel(newTimer, null);
	testModel.setNumberOfFrames(model.getNumberOfFrames());
	testModel.setDriverFactory(model.getDriverFactory());
	// just the original skeleton
  	SkeletonHandle m1=testModel.addSkeleton(original.getRootJoint());
  	m1.getDataManager().copyDriversFrom(original.getDataManager());
  	m1.getDataManager().copyOffsetsFrom(original.getDataManager());
	// displaced skeleton and offset
  	SkeletonHandle m2 = testModel.addSkeleton(displaced.getRootJoint());
  	m2.getDataManager().copyDriversFrom(displaced.getDataManager());
  	m2.getDataManager().copyOffsetsFrom(displaced.getDataManager());
	SkeletonHandle m2prime = testModel.createSkeleton(lodSkeleton.getRootJoint());
	m2prime.getDataManager().copyDriversFrom(lodSkeleton.getDataManager());
  	m2prime.getDataManager().copyOffsetsFrom(lodSkeleton.getDataManager());
	// recreate the pyramid
	HashMap newMap = new HashMap();
	newMap.put(m2.getRootJoint(), elevationMap.get(original.getRootJoint()));
	newMap.put(m2prime.getRootJoint(), elevationMap.get(((SkeletonHandle)original.getLODPyramid().getLevels().next()).getRootJoint()));
	LODPyramid clonePyramid = new LODPyramid(new DefaultLODChooser());
	clonePyramid.addLevel(m2);
	clonePyramid.addLevel(m2prime);
	m2.setLODPyramid(clonePyramid);

	// setup the view and frame
	GLModelView view = new GLModelView(cameraType);
	testModel.setMasterView(view);
	ViewFrame frame=new ViewFrame(testModel,view);
	//ViewFrame frame1=new ViewFrame(model2,new TreeModelView());
	WindowAdapter myWindowAdapter=new MyWindowAdapter();
	// frame2.addWindowListener(myWindowAdapter);
	frame.addWindowListener(myWindowAdapter);
    }
}

class MyWindowAdapter extends WindowAdapter
{
    public void windowClosing(WindowEvent e) 
    {
	System.exit(0);
    }
}
