package mc.test;

import mc.io.bvh.LODBVHParser;
import mc.io.bvh.ParseException;
import mc.model.*;
import mc.view.ViewFrame;
import mc.view.TreeModelView;
import mc.view.GLModelView;
import mc.view.Camera;
import mc.view.cameras.*;
import mc.model.ModelException;
import mc.model.euler.EulerAnglesDriverFactory;
import mc.model.quart.QuartenionDriverFactory;
import mc.model.map.MapDriverFactory;
import mc.model.DriverFactory;
import mc.util.SceneUtil;

import com.sun.java.util.collections.*;
import java.io.*;
import java.awt.event.*;
import javax.vecmath.Vector3d;

public class TestLOD2
{
    public static  double parseDouble(String str) throws NumberFormatException
    {
	return (new Double(str)).doubleValue();
    }
    
    public static final String usage = "Usage: TestLOD BVH_file";
    public static void main(String[] args) throws ModelException, IOException, ParseException, NumberFormatException
    {
	// Get the first argument and initialize the factory
	DriverFactory factory=new EulerAnglesDriverFactory();
	String cameraType = null;
	String file=null;
	
	if(args.length < 1 || args.length > 5) {
	    throw new IllegalArgumentException(usage);
	} else {
	    file=args[0];
	}
	
	// Initialize a model
	LODBVHParser parser=new LODBVHParser(new FileInputStream(file));
	parser.init();
	ModelTimer timer=new ModelTimer(33);

	BasicAnimationModel model=new BasicAnimationModel(timer);
	model.setDriverFactory(factory);
	HashMap elevationMap=new HashMap();
	parser.fill(model,elevationMap);
	parser.Hierarchy();
	// Get the original skeleton
	SkeletonHandle original=model.getFirstSkeleton();
	// Clone with a translation
	SkeletonHandle displaced=SceneUtil.cloneSkeletonSameJoints(model,original,new Vector3d(parseDouble(args[1]),parseDouble(args[2]),parseDouble(args[3])));
	// Get the pyramid and the first skeleton
	LODPyramid pyramid=displaced.getLODPyramid();
	SkeletonHandle lodSkeleton=(SkeletonHandle)pyramid.getLevels().next();
	// Clone it
	ModelTimer newTimer=new ModelTimer(33);
	BasicAnimationModel newModel=new BasicAnimationModel(newTimer);
	newModel.setNumberOfFrames(model.getNumberOfFrames());
	newModel.setDriverFactory(model.getDriverFactory());
  	SkeletonHandle clone = newModel.addSkeleton(lodSkeleton);
  	SkeletonHandle clone2=newModel.addSkeleton(original);
	// setup the view and frame
	GLModelView view = new GLModelView(cameraType);
	ViewFrame frame=new ViewFrame(newModel,view);
	//ViewFrame frame1=new ViewFrame(model2,new TreeModelView());
	WindowAdapter myWindowAdapter=new MyWindowAdapter();
	// frame2.addWindowListener(myWindowAdapter);
	frame.addWindowListener(myWindowAdapter);
    }
}

class MyWindowAdapter extends WindowAdapter
{
    public void windowClosing(WindowEvent e) 
    {
	System.exit(0);
    }
}
