package mc.view;

import mc.model.euler.EulerRotationVector3d;
import mc.model.TranslationVector3d;
import mc.model.quart.QuartenionRepresentation;
import mc.model.map.ParameterizedQuat4d;

/**
 * The data representation view
 */
public class DataViewFactory
{
    /**
     * Get the view for a specific data representation.
     *
     * @param dataDescriptionName The name of the representation.
     * @return The view
     */
    public static DataRepresentationView getDataRepresentationView(String dataDescriptionName)
    {
	if (dataDescriptionName.equals(EulerRotationVector3d.Name)) {
	    return new EulerRotationPanel();
	}
	else if (dataDescriptionName.equals(TranslationVector3d.Name)) {
	    return new TranslationVectorPanel();
	} 
	else if (dataDescriptionName.equals(QuartenionRepresentation.Name)) {
	    return new QuartenionRotationPanel();
	} 
	else if (dataDescriptionName.equals(ParameterizedQuat4d.Name)) {
	    return new ParameterizedQuatPanel();
	} else {
	    throw new IllegalArgumentException("Unsupported data representation "+dataDescriptionName);
	} 
    }
}

	
