package mc.view;

import mc.model.DataRepresentation;
import mc.model.map.ParameterizedQuat4d;
import mc.model.euler.EulerRotationDriver;
import mc.model.quart.QuartenionRepresentation;
import mc.model.euler.EulerRotationVector3d;
import javax.vecmath.Quat4d;
import javax.vecmath.AxisAngle4d;

class EulerAnglesGLView extends QuartenionGLView
{
    public void refresh(DataRepresentation t)
    {
	// Convert to a parameterized quartenion
	EulerRotationVector3d vector=(EulerRotationVector3d)t;
	if (vector.x==0 && vector.y==0 && vector.z==0) {
	    super.refresh(new QuartenionRepresentation(new Quat4d(0,0,0,1)));
	    return;
	}
	
	// Construct the three axis angles
	AxisAngle4d xRot=new AxisAngle4d(1,0,0,vector.x);
	AxisAngle4d yRot=new AxisAngle4d(0,1,0,vector.y);
	AxisAngle4d zRot=new AxisAngle4d(0,0,1,vector.z);
	// Get the rotation order
	int[] rotationOrder=vector.getRotationOrder();
	// Construct the result quartenion
	Quat4d resultQuat=new Quat4d(0,0,0,1);
	
	for(int i=0;i<rotationOrder.length;i++) {
	    AxisAngle4d rotation=null;
	    
	    switch(rotationOrder[i]) {
	    case EulerRotationDriver.X:
		rotation=xRot;
		break;
	    case EulerRotationDriver.Y:
		rotation=yRot;
		break;
	    case EulerRotationDriver.Z:
		rotation=zRot;
		break;
	    }
	    // Multiply the quartenion
	    Quat4d quat=new Quat4d();
	    quat.set(rotation);
	    resultQuat.mul(quat);
	}
	super.refresh(new QuartenionRepresentation(resultQuat));
    }
}

