package mc.view;

import mc.model.*;
import mc.util.*;

import javax.vecmath.*;
import com.sun.java.util.collections.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * The component that handles how far back we should trace
 */
final class GLCameraComponent extends JPanel implements AccelListener
{
    private static final String X = "X".intern();
    private static final String Y = "Y".intern();
    private static final String Z = "Z".intern();
    private static final String EYE = "EYE".intern();
    private static final String TGT = "TGT".intern();

    /** The model view */
    private GLModelView view;
    /** The model */
    private AnimationModel model;
    /** The view that cares about the trace depth */
    private Camera camera;
    /** Map of the button to its CameraCoord */
    private HashMap coordMap;
    /** The current step in the button acceleration */
    private double step = 1;

    /**
     * Initialize the model.
     */
    public GLCameraComponent(GLModelView mv)
    {
	super(new FlowLayout());
	//
	this.view = mv;
	this.camera = mv.getCamera();	
	this.model = mv.getModel();
	initComponents();
    }
    
    /** Initialize the visual components */
    private void initComponents()
    {
	coordMap = new HashMap();

	Vector coords = new Vector();
	coords.addElement(new CameraCoord(X, EYE,
					  new AccelButton(">"), 
					  new AccelButton("<"),
					  String.valueOf(camera.getEye(model.getCurrentFrame()).x), this) );
	coords.addElement(new CameraCoord(Y, EYE,
					  new AccelButton(">"), 
					  new AccelButton("<"),
					  String.valueOf(camera.getEye(model.getCurrentFrame()).y), this) );
	coords.addElement(new CameraCoord(Z, EYE,
					  new AccelButton(">"), 
					  new AccelButton("<"),
					  String.valueOf(camera.getEye(model.getCurrentFrame()).z), this) );
	coords.addElement(new CameraCoord(X, TGT,
					  new AccelButton(">"), 
					  new AccelButton("<"),
					  String.valueOf(camera.getTarget(model.getCurrentFrame()).x), this) );
	coords.addElement(new CameraCoord(Y, TGT,
					  new AccelButton(">"), 
					  new AccelButton("<"),
					  String.valueOf(camera.getTarget(model.getCurrentFrame()).y), this));
	coords.addElement(new CameraCoord(Z, TGT,
					  new AccelButton(">"), 
					  new AccelButton("<"),
					  String.valueOf(camera.getTarget(model.getCurrentFrame()).z), this));
	
	int cnt = 0;
	CameraCoord[] panelCoords = new CameraCoord[3];
	for(Iterator i = coords.iterator(); i.hasNext();) {
	    CameraCoord c = (CameraCoord)i.next();
	    c.register(coordMap);	    	    
	    if(panelCoords[2] != null) {
		add(coordPanel(panelCoords[0], panelCoords[1], panelCoords[2], "Eye Coordinates"));
		panelCoords = new CameraCoord[3];
		panelCoords[cnt%3] = c;
	    } else {
		panelCoords[cnt%3] = c;
	    }
	    cnt++;
	}
	add(coordPanel(panelCoords[0], panelCoords[1], panelCoords[2], "Target Coordinates"));
    }

    private JPanel coordPanel(CameraCoord x, CameraCoord y, CameraCoord z, String name)
    {
	GridBagLayout layout=new GridBagLayout();
	JPanel panel=new JPanel(layout);
	GridBagConstraints c=new GridBagConstraints();
	
	c.gridwidth=1;
	c.gridheight=1;
	c.anchor=GridBagConstraints.CENTER;
	c.weightx=c.weighty=1;
	c.insets=new Insets(0,5,0,5);
	
	// Add the x coord
	JLabel xname=new JLabel(x.name);
	layout.setConstraints(xname,c);
	panel.add(xname);
	layout.setConstraints(x.down,c);
	panel.add(x.down);
	layout.setConstraints(x.up,c);
	panel.add(x.up);
	c.gridwidth=GridBagConstraints.REMAINDER;
	layout.setConstraints(x.feedback,c);
	panel.add(x.feedback);
	// Add the y coord
	c.gridwidth=1;
	JLabel yname=new JLabel(y.name);
	layout.setConstraints(yname,c);
	panel.add(yname);
	layout.setConstraints(y.down,c);
	panel.add(y.down);
	layout.setConstraints(y.up,c);
	panel.add(y.up);
	c.gridwidth=GridBagConstraints.REMAINDER;
	layout.setConstraints(y.feedback,c);
	panel.add(y.feedback);
	// Add the z coord
	c.gridwidth=1;
	JLabel zname=new JLabel(z.name);
	layout.setConstraints(zname,c);
	panel.add(zname);
	layout.setConstraints(z.down,c);
	panel.add(z.down);
	layout.setConstraints(z.up,c);
	panel.add(z.up);
	c.gridwidth=GridBagConstraints.REMAINDER;
	layout.setConstraints(z.feedback,c);
	panel.add(z.feedback);

	panel.setBorder(BorderFactory.createTitledBorder(name));
	
	return panel;
    }

    public void stopped(ActionEvent e)
    {
	step = 1;
	update((AccelButton)(e.getSource()));
    }
    
    public void accelerated(ActionEvent e)
    {
	update((AccelButton)(e.getSource()));
	step++;
    }

    private void update(AccelButton b)
    {
	CameraCoord c = (CameraCoord)(coordMap.get(b));
	if(c != null)
	    c.update(b, deltaFunction());
	else
	    System.err.println("Shouldn't be here");
    }

    private double deltaFunction()
    {
	return step;
    }

    class CameraCoord
    {
	String name;
	String type;
	AccelButton down;
	AccelButton up;
	JTextField feedback;

	CameraCoord(String name, String type, AccelButton up, AccelButton down, String str,
		    AccelListener l)
	{
	    this.feedback = new JTextField(5);
	    feedback.setText(str);
	    feedback.setHorizontalAlignment(JTextField.RIGHT);
	    feedback.setEditable(false);
	    
	    this.down = down;
	    this.down.addAccelListener(l);
	    this.up = up;
	    this.up.addAccelListener(l);
	    this.name = name;
	    this.type = type;
	}

	void update(AccelButton button, double delta)
	{
	    double d = delta;
	    if(button == down)
		d = -d;
	    feedback.setText(String.valueOf((new Double(feedback.getText())).doubleValue() + d));
	    if(type==EYE) {
		Vector3d t = camera.getEye(model.getCurrentFrame());
		if(name==X) {		    
		    camera.setEyeX(t.x + d);
		} else if(name==Y) {
		    camera.setEyeY(t.y + d);
		} else if(name==Z) {
		    camera.setEyeZ(t.z + d);
		}
	    } else if(type==TGT) {
		Vector3d t = camera.getTarget(model.getCurrentFrame());
		if(name==X) {
		    camera.setTargetX(t.x + d);
		} else if(name==Y) {
		    camera.setTargetY(t.y + d);
		} else if(name==Z) {
		    camera.setTargetZ(t.z + d);
		}		
	    }
	    feedback.repaint();
	    view.refresh();
	}
	
	void register(HashMap map)
	{
	    map.put(down, this);
	    map.put(up, this);
	}

	JPanel getComponent()
	{
	    JPanel panel = new JPanel(new FlowLayout());
	    panel.add(new JLabel(name));
	    panel.add(down);
	    panel.add(up);
	    panel.add(feedback);

	    return panel;
	}
    }
    
}



