package mc.view;

import mc.model.AnimationModel;
import mc.model.RotationInterpolator;
import mc.model.DataManagerListener;
import mc.model.DataManagerEvent;

import com.sun.java.util.collections.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;

/**
 * The component that handles interpolations
 */
final class InterpolatorComponent extends JPanel implements ActionListener,ListSelectionListener,DataManagerListener, FocusListener, KeyListener
{
    /** The button for removing an interpolation */
    private JButton removeButton;
    
    /** The button for setting an interpolation */
    private JButton interpolateButton;
    
    /** The field for the start frame of the interpolation */
    private JTextField startField;
    
    /** The field for the end frame of the interpolation */
    private JTextField endField;
    
    /** The animation model that will be manipulated */
    private final AnimationModel theAnimationModel;
    
    /** The table model */
    private final DefaultTableModel theModel;
    
    /** The table with the interpolations */
    private final JTable theTable;

    /** The combo box that will allow the choice of the interpolator */
    private JComboBox theComboBox;
    
    /** The name of the interpolator */
    private String interpolatorName;
    
    /**
     * Initialize the model.
     */
    public InterpolatorComponent(AnimationModel model)
    {
	super(new GridBagLayout());
	// Initialize the table
	this.theTable=new JTable();
	
	//
	this.theAnimationModel=model;
	model.getFirstSkeleton().getDataManager().addListener(this);
	
	// Set the default table model and add three columns
	this.theModel=new ImmutableTableModel();
	theModel.addColumn("Start Frame");
	theModel.addColumn("End Frame");
	theModel.addColumn("Type");
	theTable.setModel(theModel);
	// Register for selections
	theTable.getSelectionModel().addListSelectionListener(this);

	// Initialize the components
	initComponents();
    }

    public void modelChanged(DataManagerEvent e)
    {
	if (e.getType()==DataManagerEvent.ROTATION_DRIVER_ADDED) {
	    // Add the row in the table
	    Object[] data={
		new Integer(e.getStartFrame()+1),
		new Integer(e.getEndFrame()+1),
		e.getDriverDescription()
	    };
	
	    theModel.insertRow(0,data);
	}
	else if (e.getType()==DataManagerEvent.ROTATION_DRIVER_REMOVED) {
	    // Iterate over the rows
	    for(int i=0;i<theModel.getRowCount();i++) {
		int start=((Integer)theModel.getValueAt(i,0)).intValue()-1;
		int end=((Integer)theModel.getValueAt(i,1)).intValue()-1;
		if (start==e.getStartFrame()&& end==e.getEndFrame()) {
		    theModel.removeRow(i);
		    break;
		}
	    }
	    removeButton.setEnabled(false);
	}
    }
    
    /** Initialize the visual components */
    private void initComponents()
    {
	GridBagConstraints c=new GridBagConstraints();
	GridBagLayout layout=(GridBagLayout)getLayout();
	
	// Create the two textfields
	GridBagLayout fieldLayout=new GridBagLayout();
	JPanel fieldPanel=new JPanel(fieldLayout);

	GridBagConstraints fc=new GridBagConstraints();
	fc.gridwidth=1;
	fc.gridheight=1;
	fc.fill=GridBagConstraints.NONE;
	fc.weightx=1;
	fc.insets=new Insets(5,5,5,5);
	fc.anchor=GridBagConstraints.CENTER;

	// Create the combo box if necessary
	String text=null;
	String[] interpolators=theAnimationModel.getDriverFactory().getInterpolatorDescriptions();
	// Assign the default interpolator
	interpolatorName=interpolators[0];
	if (interpolators.length==1) {
	    text=interpolators[0];
	} else {
	    fc.gridwidth=GridBagConstraints.REMAINDER;
	    theComboBox=new JComboBox(interpolators);
	    theComboBox.addActionListener(this);
	    fieldLayout.setConstraints(theComboBox,fc);
	    fieldPanel.add(theComboBox);
	    text="Insert";
	}
	// Add the start field
	fc.gridwidth=1;
	JLabel startLabel=new JLabel("Start: ");
	fieldLayout.setConstraints(startLabel,fc);
	fieldPanel.add(startLabel);
	startField=new JTextField(5);
	startField.addKeyListener(this);
	startField.addActionListener(this);
	startField.addFocusListener(this);
	fieldLayout.setConstraints(startField,fc);
	fieldPanel.add(startField);
	// ADd the end field
	JLabel endLabel=new JLabel("End: ");
	fieldLayout.setConstraints(endLabel,fc);
	fieldPanel.add(endLabel);
	endField=new JTextField(5);
	endField.addKeyListener(this);
	endField.addActionListener(this);
	endField.addFocusListener(this);
	fieldLayout.setConstraints(endField,fc);
	fieldPanel.add(endField);
	// Create the create button
	interpolateButton=new JButton(text);
	interpolateButton.addActionListener(this);
	interpolateButton.setEnabled(false);
	fc.gridwidth=GridBagConstraints.REMAINDER;
	fieldLayout.setConstraints(interpolateButton,fc);
	fieldPanel.add(interpolateButton);
	// Add the field
	c.gridwidth=GridBagConstraints.REMAINDER;
	c.gridheight=1;
	c.insets=new Insets(0,2,0,2);
	c.fill=GridBagConstraints.NONE;
	c.anchor=GridBagConstraints.CENTER;
	c.weightx=0.5;
	c.weighty=0.2;
	layout.setConstraints(fieldPanel,c);
	add(fieldPanel);
	
	// Add the table in the middle
	JScrollPane scroller=new JScrollPane(theTable);
	c.weighty=1;
	theTable.setPreferredScrollableViewportSize(new Dimension(100,50));
	c.fill=GridBagConstraints.BOTH;
	layout.setConstraints(scroller,c);
	add(scroller);
	
	// Add the delete button
	c.weighty=0.2;
	c.fill=GridBagConstraints.NONE;
	c.insets=new Insets(10,2,0,2);
	removeButton=new JButton("Remove");
	removeButton.addActionListener(this);
	removeButton.setEnabled(false);
	layout.setConstraints(removeButton,c);
	add(removeButton);
    }
    
    public void actionPerformed(ActionEvent e)
    {
	// Check the source of the event and branch
	if (e.getSource()==theComboBox) {
	    interpolatorName=(String)theComboBox.getSelectedItem();
	} else if (e.getSource()==interpolateButton) {
	    createInterpolation();
	}
	else if (e.getSource()==removeButton) {
	    removeInterpolation();
	} else if (e.getSource()==startField) {
	    checkInterpolateButton();
	    startField.transferFocus();
	}
	else if (e.getSource()==endField) {
	    checkInterpolateButton();
	    endField.transferFocus();
	}
    }

    public void focusLost(FocusEvent e)
    {
	checkInterpolateButton();
    }
    
    public void focusGained(FocusEvent e)
    {}
    
    /**
     * Check the state of the interpoation button, 
     * according to the text in the fields.
     *
     */
    private void checkInterpolateButton()
    {
	if (!startField.getText().equals("") && 
	    !endField.getText().equals("")) {
	    interpolateButton.setEnabled(true);
	} else {
	    interpolateButton.setEnabled(false);
	}
    }
    
    /** Create an interpolator */
    private void createInterpolation()
    {
	// Get the start Frame
	int startFrame;
	try {
	    startFrame=((new Integer(startField.getText())).intValue())-1;
	}
	catch(NumberFormatException e) {
	    showError("Could not read start frame from "+startField.getText());
	    interpolateButton.setEnabled(false);
	    return;
	}
	
	// Get the end frame
	int endFrame;
	try {
	    endFrame=((new Integer(endField.getText())).intValue())-1;
	}
	catch(NumberFormatException e) {
	    showError("Could not read end frame from "+endField.getText());
	    interpolateButton.setEnabled(false);
	    return;
	}

	// Create an interpolator
	RotationInterpolator driver=theAnimationModel.getDriverFactory().createRotationInterpolator(interpolatorName);
	driver.interpolate(startFrame,endFrame,
			   theAnimationModel.getFirstSkeleton().getDataManager().getRotationDriverFor(startFrame,endFrame));

	// Add the driver
	theAnimationModel.getFirstSkeleton().getDataManager().addRotationDriver(startFrame,endFrame,driver);
    }

    /**
     * Remove an interolation
     *
     */
    private void removeInterpolation()
    {
	// Get the selected row
	int selectedRow=theTable.getSelectedRow();

	// Check 
	if (selectedRow==-1) {
	    showError("You have to select a row first");
	    return;
	}

	// Try to get the start and end frame number
	int startFrame=((Integer)theModel.getValueAt(selectedRow,0)).intValue()-1;
	int endFrame=((Integer)theModel.getValueAt(selectedRow,1)).intValue()-1;

	// Remove it
	theAnimationModel.getFirstSkeleton().getDataManager().removeRotationDriver(startFrame,endFrame);
    }
    
    /**
     * Something was selected in the table. Make sure that 
     * the remove button is enabled.
     *
     * @param e The list selection event.
     */
    public void valueChanged(ListSelectionEvent e)
    {
	if (theTable.getSelectedRow()==-1) {
	    removeButton.setEnabled(false);
	} else {
	    removeButton.setEnabled(true);
	}
    }
    
    /**
     * Show an error dialog.
     *
     * @param message The error message
     */
    private void showError(String message)
    {
	JOptionPane.showMessageDialog(getTopLevelAncestor(),message,"Error",JOptionPane.ERROR_MESSAGE);
    }

    public void keyPressed(KeyEvent e)
    {}
    
    public void keyReleased(KeyEvent e)
    {}
    
    public void keyTyped(KeyEvent e)
    {
	checkInterpolateButton();
	interpolateButton.repaint();
    }
    
}



