package mc.view;

import mc.model.DataRepresentation;
import mc.model.quart.QuartenionRepresentation;
import mc.util.FormatBigDecimal;
import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;

/**
 * The panel that shows the value of a quartenion in tabular form.
 */
public class QuartenionRotationPanel extends JPanel implements DataRepresentationView
{
    /** The table model */
    private final DefaultTableModel theModel;

    /** The GL view */
    private final QuartenionGLView glView;
    
    /**
     * Forward an initialization message to the glView.
     *
     */
    public void initView()
    {
	glView.initView();
    }
    
    /**
     * Initialize the component
     *
     */
    public QuartenionRotationPanel()
    {
	super(new GridBagLayout());
	
	// Create the table model
	theModel=new ImmutableTableModel();
	theModel.addColumn("Channel");
	theModel.addColumn("Value");
	Object[] row=new Object[2];
	// Add the axis rows
	row[0]="Rot X Axis";
	row[1]=new Double(0);
	theModel.addRow(row);
	row[0]="Rot Y Axis";
	theModel.addRow(row);
	row[0]="Rot Z Axis";
	theModel.addRow(row);
	// Add the angle
	row[0]="Angle";
	theModel.addRow(row);
	
	// Create the table
	JTable theTable=new JTable(theModel);
	theTable.setPreferredScrollableViewportSize(new Dimension(100,50));
	JScrollPane scroller=new JScrollPane(theTable);
	// Set up the layout
	GridBagLayout layout=(GridBagLayout)getLayout();
	GridBagConstraints c=new GridBagConstraints();
	c.weightx=1;
	c.weighty=1;
	c.fill=GridBagConstraints.BOTH;
	layout.setConstraints(scroller,c);
	add(scroller);
	// Add the GL view
	glView=new QuartenionGLView();
	c.weightx=0.2;
	c.weighty=0.2;
	c.gridwidth=GridBagConstraints.REMAINDER;
	c.fill=GridBagConstraints.BOTH;
	layout.setConstraints(glView,c);
	add(glView);
	//
	setBorder(BorderFactory.createTitledBorder("Rotation"));
    }
    
    public Component getComponent()
    {
	return this;
    }
    
    public void refresh(DataRepresentation rep)
    {
	QuartenionRepresentation quat=(QuartenionRepresentation)rep;
	// Update
	theModel.setValueAt(FormatBigDecimal.format(quat.x,3),0,1);
	theModel.setValueAt(FormatBigDecimal.format(quat.y,3),1,1);
	theModel.setValueAt(FormatBigDecimal.format(quat.z,3),2,1);
	theModel.setValueAt(FormatBigDecimal.format(quat.w,3),3,1);

	// Refresh the gl view
	glView.refresh(rep);
    }
}

	
