package mc.view;

import com.sun.java.util.collections.HashMap;
import com.sun.java.util.collections.Iterator;
import com.sun.java.util.collections.Vector;
import mc.model.Joint;
import mc.util.Pair;
import mc.view.shapes.*;

/**
 * Keeps track of adapter to shape mappings. An adapter
 * may have possibly more that one shape mapped to it.
 */
public class ShapeManager
{
    public final static Shape DEFAULT_JOINT_SHAPE = new ConeShape(4, 2);
    public final static Shape DEFAULT_SEGMENT_SHAPE = new RectangleShape();

    private HashMap jointShapeMap; // key:Joint, val:Vector of Shape
    private HashMap segmentShapeMap; // key:Pair of Joints, val:Vector of Shape

    public ShapeManager()
    {
	jointShapeMap = new HashMap();
	segmentShapeMap = new HashMap();
    }

    /**
     * Produce the set of shapes associated with a given joint adapter
     * @param joint adapter
     * @return iterator over set of shapes
     * @see JointAdapter
     */
    public Iterator getJointShapes(JointAdapter a)
    {
	Vector v = (Vector)jointShapeMap.get(a.getJoint());
	if(v==null)
	    v = new Vector();
	return v.iterator();
    }

    /**
     * Produce the set of shapes associated with a given segment adapter
     * @param segment adapter
     * @return iterator over set of shapes
     * @see SegmentAdapter
     */
    public Iterator getSegmentShapes(SegmentAdapter a)
    {
	Vector v = (Vector)segmentShapeMap.get(new Pair(a.getParent(), a.getChild()));
	if(v==null)
	    v = new Vector();
	return v.iterator();
    }

    /**
     * Add a shape to the association of shapes to a given JointAadpter
     * @param joint adapter
     * @see JointAdapter
     * @see Shape
     */
    public void addShapeToJoint(Joint a, Shape s)
    {
	Vector v = (Vector)jointShapeMap.get(a);
	if(v==null) {
	    v = new Vector();
	    jointShapeMap.put(a,v);
	}
	v.addElement(s);
    }

    /**
     * Add a shape to the association of shapes to a given SegmentAadpter
     * @param joint adapter
     * @see SegmentAdapter
     * @see Shape
     */
    public void addShapeToSegment(Joint p, Joint c, Shape s)
    {
	Vector v = (Vector)segmentShapeMap.get(new Pair(p,c));
	if(v==null) {
	    v = new Vector();
	    segmentShapeMap.put(new Pair(p,c), v);
	}
	v.addElement(s);
    }
}
