package mc.view;

import mc.model.AnimationModel;
import mc.model.ModelListener;
import mc.model.ModelEvent;
import mc.util.Pair;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import com.sun.java.util.collections.*;


/**
 * A frame that visualizes an animation model.
 */
public class ViewFrame extends JFrame implements ModelListener, ActionListener, ItemListener
{
    private static final String LOOP_ICON_NAME="loop.gif";
    
    private static final String PREV_ICON_NAME="previous.gif";
    
    private static final String NEXT_ICON_NAME="next.gif";

    private static final String PLAY_ICON_NAME="play.gif";
    
    private static final String PLAYBACK_ICON_NAME="playb.gif";
    
    private static final String STOP_ICON_NAME="stop.gif";
    
    private static final String FF_ICON_NAME="ff.gif";
    
    private static final String RW_ICON_NAME="rw.gif";
    
    /**
     * The loop button.
     */
    private JCheckBox loopButton;

    /** The map that carries infomation on buttons */
    private final Map buttonMap;
    
    /** the actual view */
    private final ModelView modelView;

    /** the model */
    private final AnimationModel model;
    
    /** The frame field */
    private JTextField frameField;
    
    /** The prev button */
    private JButton prevButton;
    
    /** The next button */
    private JButton nextButton;

    /** The play button */
    private JButton playButton;
    
    /** The play backwards button */
    private JButton playBackButton;
    
    /** The button for fast forward */
    private JButton ffButton;
    
    /** The button for rewind */ 
    private JButton rwButton;

    /** The button that has trigerred the last animation */
    private JButton animButton;
    
    /**
     * Initialize the frame
     *
     */
    public ViewFrame(AnimationModel model,ModelView view)
    {
	super();
	this.model=model;
	this.modelView=view;
	this.buttonMap=new HashMap();
	
	view.setModel(model);
	model.addModelListener(this);

	// Init the visual components
	initComponents();

	setSize(200,200);
	pack();
	setVisible(true);
	setTitle("Model View");
	
	view.initView();
	// Set the frame number to 0
	model.setFrame(0);
    }
    
    private void initComponents()
    {
	// Get the layout manager
	GridBagLayout layout=new GridBagLayout();
	super.getContentPane().setLayout(layout);
	
	// Add the view component in the middle
	GridBagConstraints c=new GridBagConstraints();
	c.insets=new Insets(2,5,2,5);
	c.gridwidth=1;
	c.gridheight=3;
	c.weightx=1;
	c.weighty=1;
	c.fill=GridBagConstraints.BOTH;
	c.anchor=GridBagConstraints.CENTER;
	Component component=modelView.getComponent();
	JPanel componentPanel=new JPanel(new BorderLayout());
	componentPanel.add(component);
	componentPanel.setBorder(BorderFactory.createTitledBorder("View"));
	layout.setConstraints(componentPanel,c);
	getContentPane().add(BorderLayout.CENTER,componentPanel);

	// Add the right panel
	c.gridheight=1;
	c.gridwidth=GridBagConstraints.REMAINDER;
	c.gridx=1;
	c.gridy=0;
	c.fill=GridBagConstraints.BOTH;
	c.weightx=0.5;
	c.weighty=1.0;
	JPanel rightPanel=new JPanel(new BorderLayout());
	rightPanel.setBorder(BorderFactory.createTitledBorder("Interpolate/Filter"));
	layout.setConstraints(rightPanel,c);
	getContentPane().add(rightPanel);

	// The interpolator
	InterpolatorComponent interpolatorComponent=new InterpolatorComponent(model);
	rightPanel.add(BorderLayout.CENTER,interpolatorComponent);
	
	c.gridx=1;
	c.gridy=1;
	c.fill=GridBagConstraints.BOTH;
	c.weightx=0.5;
	c.weighty=1;
	component=modelView.getControlComponent();
	layout.setConstraints(component,c);
	getContentPane().add(component);
	
	// Create the time panel
	GridBagLayout timeLayout=new GridBagLayout();
	GridBagConstraints timec=new GridBagConstraints();
	JPanel timePanel=new JPanel(timeLayout);
	
	// Create the button panel
	timec.fill=GridBagConstraints.BOTH;
	timec.anchor=GridBagConstraints.CENTER;
	timec.gridwidth=GridBagConstraints.REMAINDER;
	timec.insets=new Insets(0,5,0,5);
	JPanel buttonPanel=new JPanel(new GridLayout(0,6));
	timeLayout.setConstraints(buttonPanel,timec);
	timePanel.add(buttonPanel);
	
	// Add the rewind button 
	ImageIcon rwIcon=new ImageIcon(RW_ICON_NAME);
	rwButton=new JButton(rwIcon);
	rwButton.addActionListener(this);
	rwButton.setActionCommand("Rewind");
	rwButton.setEnabled(false);
	buttonPanel.add(rwButton);
	buttonMap.put(rwButton,new Pair(rwIcon,"Rewind"));

	// Add the previous button
	prevButton=new JButton(new ImageIcon(PREV_ICON_NAME));
	prevButton.addActionListener(this);
	prevButton.setActionCommand("Prev");
	prevButton.setEnabled(false);
	buttonPanel.add(prevButton);
	
	// Add the playb button 
	ImageIcon playBackIcon=new ImageIcon(PLAYBACK_ICON_NAME);
	playBackButton=new JButton(playBackIcon);
	playBackButton.addActionListener(this);
	playBackButton.setActionCommand("PlayBack");
	playBackButton.setEnabled(false);
	buttonPanel.add(playBackButton);
	buttonMap.put(playBackButton,new Pair(playBackIcon,"PlayBack"));
	
	// The play button
	ImageIcon playIcon=new ImageIcon(PLAY_ICON_NAME);
	playButton=new JButton(playIcon);
	playButton.addActionListener(this);
	playButton.setActionCommand("Play");
	buttonPanel.add(playButton);
	buttonMap.put(playButton,new Pair(playIcon,"Play"));
	
	// The next button
	nextButton=new JButton(new ImageIcon(NEXT_ICON_NAME));
	nextButton.addActionListener(this);
	nextButton.setActionCommand("Next");
	buttonPanel.add(nextButton);
	
	// The ff button
	ImageIcon ffIcon=new ImageIcon(FF_ICON_NAME);
	ffButton=new JButton(ffIcon);
	ffButton.addActionListener(this);
	ffButton.setActionCommand("FF");
	buttonPanel.add(ffButton);
	buttonMap.put(ffButton,new Pair(ffIcon,"FF"));
	
	// Add the frame label
	JPanel framePanel=new JPanel();
	framePanel.add(new JLabel("Frame:"));
	// Add the frame field
	frameField=new JTextField(5);
	frameField.setHorizontalAlignment(JTextField.RIGHT);
	frameField.addActionListener(this);
	framePanel.add(frameField);
	// Add the total frames
	framePanel.add(new JLabel("/"+model.getNumberOfFrames()));
	
	// Initialize the checkbox
	loopButton=new JCheckBox("Enable Looping");
	loopButton.addItemListener(this);
	
	// Add the panel
	timec.gridwidth=1;
	timec.fill=GridBagConstraints.HORIZONTAL;
	timec.weightx=1;
	timec.weighty=1;
	timec.anchor=GridBagConstraints.EAST;
	timeLayout.setConstraints(framePanel,timec);
	timePanel.add(framePanel);
	// Add the button
	timec.anchor=GridBagConstraints.WEST;
	timec.gridwidth=GridBagConstraints.REMAINDER;
	timeLayout.setConstraints(loopButton,timec);
	timePanel.add(loopButton);
	// Set the border
	timePanel.setBorder(BorderFactory.createTitledBorder("Time"));

	// Add the time panel
	c.gridx=1;
	c.gridy=2;
	c.gridwidth=GridBagConstraints.REMAINDER;
	c.gridheight=GridBagConstraints.REMAINDER;
	c.fill=GridBagConstraints.HORIZONTAL;
	c.anchor=GridBagConstraints.SOUTH;
	c.weightx=0.5;
	c.weighty=0;
	layout.setConstraints(timePanel,c);
	getContentPane().add(timePanel);
    }
    
    /**
     * The buttons have been pressed.
     *
     */
    public void actionPerformed(ActionEvent e)
    {
	if (e.getSource()==frameField) {
	    // Read the field
	    int newFrame=0;
	    try {
		newFrame=Integer.parseInt(frameField.getText())-1;
	    }
	    catch(NumberFormatException error) {
		frameField.setText(Integer.toString(model.getCurrentFrame()));
		return;
	    }
	    // Check that the new frame is within the bounds
	    if (newFrame<=0 || newFrame>=model.getNumberOfFrames()) {
		frameField.setText(Integer.toString(model.getCurrentFrame()+1));
	    }
	    // Set the new frame
	    model.tick(newFrame-model.getCurrentFrame());
	}
	if (e.getActionCommand().equals("Prev")) {
	    model.tick(-1);
	}
	else if (e.getActionCommand().equals("Next")) {
	    model.tick(1);
	}
	else if (e.getActionCommand().equals("Play")) {
	    model.play(1);
	    setUpPlay(playButton);
	}
	else if (e.getActionCommand().equals("FF")) {
	    model.play(10);
	    setUpPlay(ffButton);
	}
	else if (e.getActionCommand().equals("Rewind")) {
	    model.play(-10);
	    setUpPlay(rwButton);
	}
	else if (e.getActionCommand().equals("PlayBack")) {
	    model.play(-1);
	    setUpPlay(playBackButton);
	}
	else if (e.getActionCommand().equals("Pause")) {
	    model.stop();
	}
    }

    /**
     * Update the buttons based on the current frame
     * and the looping behavior of the model
     *
     */
    private void updateButtons()
    {
	if (model.getCurrentFrame()==0) {
	    playBackButton.setEnabled(model.isLooping());
	    prevButton.setEnabled(model.isLooping());
	    rwButton.setEnabled(model.isLooping());
	} else {
	    prevButton.setEnabled(true);
	    rwButton.setEnabled(true);
	    playBackButton.setEnabled(true);
	}
	
	if (model.getCurrentFrame()==model.getNumberOfFrames()-1) {
	    nextButton.setEnabled(model.isLooping());
	    ffButton.setEnabled(model.isLooping());
	    playButton.setEnabled(model.isLooping());
	} else {
	    nextButton.setEnabled(true);
	    ffButton.setEnabled(true);
	    playButton.setEnabled(true);
	}
    }

    /**
     * The animation has stopped. The button that
     * has triggered the play must be restored to 
     * the previous state.
     */
    private void stopAnimation()
    {
	if (animButton==null)
	    return;
	
	// Get the info
	Pair info=(Pair)buttonMap.get(animButton);
	// Set up the icon
	animButton.setIcon((ImageIcon)info.first);
	// Set up the action
	animButton.setActionCommand((String)info.second);

	//
	animButton=null;
    }
    
    /**
     * Set up a button so that it can pause the animation.
     *
     * @param button The button.
     */
    private void setUpPlay(JButton button)
    {
	button.setIcon(new ImageIcon(STOP_ICON_NAME));
	button.setActionCommand("Pause");
	animButton=button;
    }
	    
    /**
     * The model has been modified. Update the view.
     */
    public void modelModified(ModelEvent o)
    {
	if (o.getType()==ModelEvent.ANIMATION_STOPPED) {
	    stopAnimation();
	} else if (o.getType()==ModelEvent.LOOP_CHANGED) {
	    updateButtons();
	}
    }

    public void frameTicked(int currentFrame,int increment)
    {
	// Update the frame field
	frameField.setText(Integer.toString(model.getCurrentFrame()+1));
	// Upate the view
	modelView.refresh();
	// Update the buttons
	updateButtons();
    }
    
    /** An event from the checkbox */
    public void itemStateChanged(ItemEvent e)
    {
	if (e.getSource()==loopButton) {
	    model.setLoop(e.getStateChange()==ItemEvent.SELECTED);
	}
    }
}

