package mc.view.cameras;

import javax.vecmath.Matrix4d;
import javax.vecmath.Vector3d;
import javax.vecmath.Point3d;
import com.sun.java.util.collections.HashMap;
import com.sun.java.util.collections.Iterator;
import mc.model.AnimationModel;
import mc.model.Joint;
import mc.view.BoundingVolume;
import mc.view.BasicShapeAdapter;
import mc.view.shapes.WireCubeShape;
import mc.util.Pair;
import mc.util.SceneUtil;
import com.hermetica.magician.*;

/**
 * Automated camera control
 */
public class AutoCamera extends ManualCamera
{
    private AnimationModel model;
    private Director director;
    private int lastDecision;

    public AutoCamera(Director director, AnimationModel model)
    {
	this.model = model;
	this.director = director;
	this.lastDecision = 0;
    }

    public Vector3d getEye(int frame)
    {
	updateCamera(frame);
	return super.getEye(frame);
    }

    public Vector3d getTarget(int frame)
    {
	updateCamera(frame);
	return super.getTarget(frame);
    }

    public void hireNewDirector(Director director)
    {
	this.director = director;
    }

    private void updateCamera(int frame)
    {
	if(lastDecision != frame) {
	    director.decide(this, model, frame);
	    lastDecision = frame;
	}
    }

    public void draw(AnimationModel model, GL gl)
    {
	Pair p = SceneUtil.getBoundingVolume(model, model.getCurrentFrame());
	BoundingVolume modelVolume = (BoundingVolume)p.first;	    
	Vector3d modelCenter = (Vector3d)p.second;
	Matrix4d m = new Matrix4d();
	m.setIdentity();
	m.set(modelCenter);
	Vector3d debugEye = getEye(model.getCurrentFrame());
	Vector3d debugTgt = getTarget(model.getCurrentFrame());
	BasicShapeAdapter a = new BasicShapeAdapter(modelVolume, m);
	
	// draw bounding volume
	//System.out.println(modelVolume);
	gl.glPushMatrix();
	gl.glMultMatrixd(SceneUtil.toGLMatrix(a.getLocalMatrix()));
	(new WireCubeShape()).draw(a,gl);
	gl.glPopMatrix();
	// draw camera to target line segment
	gl.glBegin(GL.GL_LINE_STRIP);
	gl.glVertex4d(debugEye.x, debugEye.y, debugEye.z, 1.0);
	gl.glVertex4d(debugTgt.x, debugTgt.y, debugTgt.z, 1.0);
	gl.glEnd();
    }
}


