package mc.view.cameras;

import javax.vecmath.Vector3d;
import javax.vecmath.Point3d;
import javax.vecmath.Matrix4d;
import com.sun.java.util.collections.HashMap;
import mc.util.Pair;
import mc.util.SceneUtil;
import mc.model.AnimationModel;
import mc.view.Camera;
import mc.model.SkeletonHandle;

public class ChoppyDirector extends Director
{
    private double distance = 150;
    public ChoppyDirector(String name, int priorDepth, int futureDepth)
    {
	super(name, priorDepth, futureDepth);
    }
    
    protected CameraInfo collectCameraInfo(AnimationModel model, int frame)
    {
	// let the reference point be based on the first model seen
	SkeletonHandle skeleton = model.getFirstSkeleton();
	// target is always the position of the root joint
	Matrix4d tgtMatrix = SceneUtil.findGlobalJointMatrix(skeleton, new HashMap(),
							     skeleton.getRootJoint(),frame);
	//System.out.println(tgtMatrix);
	Point3d v = new Point3d();
	tgtMatrix.transform(v);
	Vector3d tgtVector = new Vector3d(v.x,v.y,v.z);
	// eye is based on extending orientation a given distance
	Vector3d orientation = SceneUtil.getJointOrientation(skeleton, skeleton.getRootJoint(), frame);
	orientation.scaleAdd(distance, tgtVector);
	//System.out.println(orientation + "," + tgtVector);
	return new ChoppyCameraInfo(orientation, tgtVector);
    }

    protected Pair makeDecision(AnimationModel model, int frame)
    {
	Pair p = new Pair();
	ChoppyCameraInfo info = (ChoppyCameraInfo)getFutureKnowledge().getInfo(frame);
	p.first = info.getEye();
	p.second = info.getTgt();

	return p;
    }

    class ChoppyCameraInfo implements CameraInfo
    {
	private Vector3d tgt;
	private Vector3d eye;

	public ChoppyCameraInfo(Vector3d eye, Vector3d tgt)
	{
	    this.tgt = tgt;
	    this.eye = eye;
	}

	public Vector3d getTgt()
	{
	    return tgt;
	}

	public Vector3d getEye()
	{
	    return eye;
	}
    }
}
