package mc.view.cameras;

import javax.vecmath.Vector3d;
import com.sun.java.util.collections.HashMap;
import mc.util.Pair;
import mc.model.AnimationModel;
import mc.view.Camera;

// TODO:
// 1. allow the depth for each memory to be reset
public abstract class Director
{
    private String name;
    private Knowledge before;
    private Knowledge after;

    public Director(String name, int priorDepth, int futureDepth)
    {
	this.name = name;
	this.before = new Knowledge(-priorDepth, priorDepth);
	this.after = new Knowledge(0, futureDepth);
    }

    public void decide(Camera camera, AnimationModel model, int frame)
    {
	int now = after.getStart();
	before.shiftStart(frame - now);
	after.shiftStart(frame - now);
	for(int i = before.getStart(); i < before.getStart() + before.getDepth(); i++) {
	    if(before.getInfo(i) == null)
		before.setInfo(collectCameraInfo(model, i), i);
	}
	for(int i = after.getStart(); i < after.getStart() + after.getDepth(); i++) {
	    if(after.getInfo(i) == null)
		after.setInfo(collectCameraInfo(model, i), i);
	}
	Pair c = makeDecision(model, frame);
	Vector3d e = (Vector3d)c.first;
	Vector3d t = (Vector3d)c.second;
	camera.setEyeX(e.x);
	camera.setEyeY(e.y);
	camera.setEyeZ(e.z);
	camera.setTargetX(t.x);
	camera.setTargetY(t.y);
	camera.setTargetZ(t.z);
    }

    public String getName()
    {
	return name;
    }

    protected abstract CameraInfo collectCameraInfo(AnimationModel model, int frame);

    protected abstract Pair makeDecision(AnimationModel model, int frame);

    protected Knowledge getPriorKnowledge()
    {
	return before;
    }

    protected Knowledge getFutureKnowledge()
    {
	return after;
    }

    class Knowledge
    {
	private int start;
	private int depth;
	private HashMap memory;

	public Knowledge(int start, int depth)
	{
	    this.start = start;
	    this.depth = depth;
	    this.memory = new HashMap();
	    for(int i = start; i < start+depth; i++) {
		memory.put(new Integer(i), null);
	    }
	}

	public void setInfo(CameraInfo info, int frame)
	{
	    memory.put(new Integer(frame), info);
	}

	public CameraInfo getInfo(int frame)
	{
	    return (CameraInfo)(memory.get(new Integer(frame)));
	}

	public void shiftStart(int offset)
	{
	    int newStart = start+offset;
	    int startKill = start;
	    int stopKill = startKill + offset;
	    int startAdd = start + depth;
	    int stopAdd = newStart + depth;
	    if(newStart < start) {
		startKill = newStart + depth;
		stopKill = depth;
		startAdd = newStart;
		stopAdd = start;
	    }
	    // clear old memory...
	    for(int i = startKill; i < stopKill; i++) {
		memory.remove(new Integer(i));
	    }
	    // ...to be replaced with new information
	    for(int i = startAdd; i < stopAdd; i++) {
		memory.put(new Integer(i), null);
	    }
	    start = newStart;
	}
	
	public int getStart()
	{
	    return start;
	}

	public int getDepth()
	{
	    return depth;
	}
    }
}
