package mc.view.shapes;

import mc.view.*;
import com.hermetica.magician.GL;
import com.hermetica.util3d.*;
import java.awt.Color;

public class CheckeredPlaneShape extends AbstractShape
{
    /** The checkerboard data */
    private int checkImageWidth = 64,
                checkImageHeight = 64;
    private byte[] checkImage = new byte[checkImageWidth * checkImageHeight * 3];

    /** Texture IDs */
    int[] texName = new int[1];

    /** Texture coordinates */
    private float textureFactor = 1.0f;

    /**
     * Create a rectangle shape.
     */
    public CheckeredPlaneShape(Color on, Color off, GL gl)
    {
	super();
	
	gl.glShadeModel(GL.GL_FLAT);
	makeCheckImage(on, off);
	gl.glPixelStorei( GL.GL_UNPACK_ALIGNMENT, 1 );
	
	/** Create a new texture */
	gl.glGenTextures( 1, texName );
	gl.glBindTexture( GL.GL_TEXTURE_2D, texName[0] );
	gl.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S,
			     GL.GL_REPEAT );
	gl.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T,
			     GL.GL_REPEAT );
	gl.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER,
			     GL.GL_NEAREST );
	gl.glTexParameteri( GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER,
			     GL.GL_NEAREST );
	gl.glTexImage2D( GL.GL_TEXTURE_2D, 0, GL.GL_RGB, 
			  checkImageWidth, checkImageHeight,
			  0, GL.GL_RGB, GL.GL_UNSIGNED_BYTE,
			  checkImage );
	gl.glHint( GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST );
    }

    public void draw(ShapeAdapter s, GL gl)
    {
	BoundingVolume v = s.getBoundingVolume();
	
	double min_x = -v.getLength()/2.0;
	double max_x =  v.getLength()/2.0;
	double min_y = -v.getHeight()/2.0;
	double max_y =  v.getHeight()/2.0;
	double min_z = -v.getWidth()/2.0;
	double max_z =  v.getWidth()/2.0;
	gl.glEnable( GL.GL_TEXTURE_2D );
	gl.glTexEnvf( GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE,
		      GL.GL_DECAL );
	gl.glBindTexture( GL.GL_TEXTURE_2D, texName[0] );	

	gl.glBegin( GL.GL_QUADS );
	  gl.glTexCoord2f( 0.0f, 0.0f ); gl.glVertex3d( min_x, 0.0, max_z );
	  gl.glTexCoord2f( 0.0f, textureFactor ); gl.glVertex3d( min_x, 0.0, min_z );
	  gl.glTexCoord2f( textureFactor, textureFactor ); gl.glVertex3d( max_x, 0.0, min_z );
	  gl.glTexCoord2f( textureFactor, 0.0f ); gl.glVertex3d( max_x, 0.0, max_z );
	gl.glEnd();	
	gl.glDisable(GL.GL_TEXTURE_2D);
    }
    
    private void makeCheckImage(Color on, Color off) 
    {
	Color c = on;
        int index = 0;
        for ( int i = 0 ; i < checkImageHeight ; i++ ) {
            for ( int j = 0 ; j < checkImageWidth ; j++ ) {
                boolean tmp = ( ( ( ( i & 0x8 ) == 0 ) ^ ( ( j & 0x8 ) ) == 0 ) );
                if ( tmp ) {
                    c = on;
		} else {
		    c = off;
		}
                //c *= 255;
                checkImage[index] = (byte)c.getRed();
                checkImage[index + 1] = (byte)c.getGreen();
                checkImage[index + 2] = (byte)c.getBlue();
                index += 3;
	    }
	}
    }    
}
