
package mc.view.shapes;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import com.hermetica.magician.*;
import com.hermetica.util3d.shapes;
import com.hermetica.util3d.*;
import mc.view.*;

public class TestShape extends Frame implements demoTemplate, GLEventListener, KeyListener, WindowListener {

    /** The OpenGL state machine */
    private CoreGL gl_ = new CoreGL();
    private CoreGLU glu_ = new CoreGLU();

    /** The OpenGL component to render onto */
    private GLComponent glc = null;

    /** The dimensions of the component */
    private static final int WIDTH = 500,
                             HEIGHT = 500;

    /** Regulates the calling of System.exit() */
    private boolean callExit = true;

    public static void main( String argv[] ) {
        TestShape d = new TestShape();
        d.setTitle( d.getName() );
        System.err.println( "\n" + d.getName() + " -- " + d.getOriginalAuthor() );
        for ( int i = 0 ; i < ( d.getName().length() + d.getOriginalAuthor().length() + 4 ) ; i++ ) {
            System.err.print( "=" );
          }
        System.err.println( "\n" );
        System.err.println( d.getDescription() + "\n" );
        d.startDemo();
      }

    public TestShape() {
        /**#start */
      }

    /** Set to false if the demo is to call System.exit() */
    public void setExit( boolean state ) {
        callExit = state;
      }

    /** Returns the name of the demo */
    public String getName() {
        return new String( "Cube" );
      }

    /** Returns the version information of this demo */
    public String getVersion() {
        return new String( "$Id: TestShape.java,v 1.4 2000/05/10 13:22:57 vuk Exp $" );
      }

    /** Returns the meaningful description of what this demo does */
    public String getDescription() {
        StringBuffer sb = new StringBuffer();
        sb.append( "This demonstration program renders a wireframe cube using\n" );
        sb.append( "the wireCube() method defined in the CoreGLU class.\n\n" );
        sb.append( "Hit the ``Escape'' key to destroy the demo." );
        return new String( sb.toString() );
      }

    /** Returns the width of the demo */
    public int getWidth() {
        return WIDTH;
      }

    /** Returns the height of the demo */
    public int getHeight() {
        return HEIGHT;
      }

    /** Returns the original author of this demo */
    public String getOriginalAuthor() {
        return new String( "Mason Woo, Tom Davis, Jackie Neider" );
      }

    public void startDemo() {

        /** Set the layout of the frame */
        setLayout( new BorderLayout() );

        glc = (GLComponent)GLComponentFactory.createGLComponent( WIDTH, HEIGHT );

        /** Add the Component into the window */
        add( "Center", glc );
        pack(); show();

        /** Setup the context capabilities */
        GLCapabilities cap = glc.getContext().getCapabilities();
        cap.setDepthBits( 12 );
        cap.setPixelType( GLCapabilities.RGBA );
        cap.setColourBits( 24 );
        cap.setDoubleBuffered( GLCapabilities.DOUBLEBUFFER );

        /** Register the GLEventListener with the GLComponent */
        glc.addGLEventListener( this );

        /** Register the keyboard listener */
        glc.addKeyListener( this );

        this.addWindowListener( this );

        /** Initialize the component */
        glc.initialize();
      }

    /** Initialization stuff */
    public void initialize( GLDrawable component ) {
        try {
            gl_.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
            gl_.glShadeModel( GL.GL_FLAT );
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Handles viewport resizing */
    public void reshape( GLDrawable component, int x, int y, int width, int height ) {
        /** Setup the viewport */
        gl_.glViewport( component, x, y, width, height );
        gl_.glMatrixMode( GL.GL_PROJECTION );
        gl_.glLoadIdentity();
        gl_.glFrustum( -1.0, 1.0, -1.0, 1.0, 1.5, 20.0 );
        gl_.glMatrixMode( GL.GL_MODELVIEW );
      }

    /** Draws a single line */
    public void display( GLDrawable component ) {
        try {
            /** Draw the cube */
            gl_.glClear( GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
	    gl_.glEnable(GL.GL_DEPTH_TEST);
            gl_.glColor3f( 1.0f, 1.0f, 1.0f );
            gl_.glLoadIdentity();
            glu_.gluLookAt( 0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 );
            //gl_.glScalef( 1.0f, 2.0f, 1.0f );
            //shapes.wireCube( 1.0 );
	    //RectangleShape r = new RectangleShape();
	    EllipsoidShape e = new EllipsoidShape(50,50);
	    //gl_.glRotatef(45,1,0,0);
	    //gl_.glRotatef(45,0,1,0);
	    //gl_.glRotatef(90,1,0,0);
	    e.draw(new TestAdapter(new BoundingVolume(1.0, 0.5, 0.5)), gl_);
          } catch ( OpenGLException e ) {
            e.printStackTrace();
          }
      }

    /** Returns a GL pipeline from the listener */
    public GL getGL() {
        return gl_;
      }

    /** Handles keyboard events */
    /** Handles key pressed event */
    public void keyPressed( KeyEvent evt ) {
        if ( evt.getKeyChar() == KeyEvent.VK_ESCAPE ) {
            /** Destroy the GLContext */
            glc.destroy();
            dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

          }
      }

    /** Handles key typed events */
    public void keyTyped( KeyEvent evt ) {
      }

    /** Handles key released events */
    public void keyReleased( KeyEvent evt ) {
      }

    /** WindowListener methods */
    public void windowActivated( WindowEvent evt ) {
      }

    public void windowClosed( WindowEvent evt ) {
      }

    public void windowClosing( WindowEvent evt ) {
        glc.destroy();
        dispose();
        if ( callExit ) {
            System.exit( 0 );
          }

      }

    public void windowDeactivated( WindowEvent evt ) {
      }

    public void windowDeiconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    public void windowIconified( WindowEvent evt ) {
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    public void windowOpened( WindowEvent evt ) {
      }
    /** Applet only */
    public void stop() {
//        System.err.println( "stop() called" );
        /** Suspend the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.suspend();
          }
      }

    /** Applet only */
    public void start() {
//        System.err.println( "start()" );
        /** Restart the GL component only if it's driving an event loop */
        if ( glc.isRunning() ) {
            glc.resume();
          }
      }

    /** Applet only */
    public void destroy() {
//        System.err.println( "destroy() called" );
        /** Destroy the GL component to free up the resources */
        glc.destroy();
      }
  }
